package com.ebei.screen.common.aspect;

import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 通过切面 记录系统日志
 *
 * @author Levi
 */
@Slf4j
@Aspect
@Component
public class SysLogAspect {

    private final ThreadLocal<LocalDateTime> currentThread = new ThreadLocal<>();

    @Pointcut("@annotation(io.swagger.annotations.ApiOperation)")
    public void logPointCut() {
    }

    @Before("logPointCut() && @annotation(apiOperation)")
    public void invokeBefore(JoinPoint joinPoint, ApiOperation apiOperation) {
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = requestAttributes.getRequest();
        log.info("\033[1;33m========================================" + Thread.currentThread().getName() + "请求开始========================================\033[0m");
        try {
            log.info("\033[1;33m请求地址:<" + request.getMethod() + "> " + request.getRequestURL().toString() + "\033[0m");
            log.info("\033[1;33m请求描述:" + apiOperation.value() + "\033[0m");
            log.info("\033[1;33m请求方法:" + joinPoint.getSignature() + "\033[0m");
            if ("GET".equals(request.getMethod())) {
                log.info("\033[1;33m请求参数:" + JSONObject.toJSONString(request.getQueryString() == null ? "无请求参数" : request.getQueryString().replace("&", "、")) + "\033[0m");
            } else {
                List<Object> args = Arrays.stream(joinPoint.getArgs()).filter(x -> !String.valueOf(x).contains("org.apache")).collect(Collectors.toList());
                log.info("\033[1;33m请求参数:" + JSONObject.toJSONString(args) + "\033[0m");
            }
            currentThread.set(LocalDateTime.now());
        } catch (Exception e) {
            log.info("\033[1;33m日志切面出现错误:", e + "\033[0m");
        }
    }

    @AfterReturning(returning = "obj", pointcut = "logPointCut()")
    public void invokeAfter(Object obj) {
        if (currentThread.get() != null) {
            //log.info("\033[1;33m响应参数:" + (obj == null ? "无响应参数" : JSONObject.toJSONString(obj)) + "\033[0m");
            log.info("\033[1;33m本次耗时:" + Duration.between(currentThread.get(), LocalDateTime.now()).toMillis() + "毫秒\033[0m");
            log.info("\033[1;33m========================================" + Thread.currentThread().getName() + "响应结束========================================\033[0m");
        }
    }
}

