package com.ebei.screen.common.config;

import com.github.xiaoymin.knife4j.spring.annotations.EnableKnife4j;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.web.context.WebServerInitializedEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import springfox.bean.validators.configuration.BeanValidatorPluginsConfiguration;
import springfox.documentation.builders.ApiInfoBuilder;
import springfox.documentation.builders.PathSelectors;
import springfox.documentation.builders.RequestHandlerSelectors;
import springfox.documentation.service.ApiInfo;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.swagger2.annotations.EnableSwagger2;

import java.net.Inet4Address;
import java.net.InetAddress;

/**
 * Knife4j配置
 */
@Slf4j
@Configuration
@EnableSwagger2
@EnableKnife4j
@Import(BeanValidatorPluginsConfiguration.class)
public class Knife4jConfig implements ApplicationListener<WebServerInitializedEvent> {

    private boolean isOpen = false;

    @Bean(value = "videoMonitor")
    public Docket videoMonitor() {
        return new Docket(DocumentationType.SWAGGER_2)
                .enable(isOpen)
                .useDefaultResponseMessages(false)
                .apiInfo(apiInfo())
                .groupName("视频监控大屏")
                .select()
                .apis(RequestHandlerSelectors.basePackage("com.ebei.screen.controller.video"))
                .paths(PathSelectors.any())
                .build();

    }

    @Bean(value = "parkSystem")
    public Docket parkSystem() {
        return new Docket(DocumentationType.SWAGGER_2)
                .enable(isOpen)
                .apiInfo(apiInfo())
                .groupName("车场系统大屏")
                .select()
                .apis(RequestHandlerSelectors.basePackage("com.ebei.screen.controller.park"))
                .paths(PathSelectors.any())
                .build();
    }

    @Bean(value = "ebaSystem")
    public Docket ebaSystem() {
        return new Docket(DocumentationType.SWAGGER_2)
                .enable(isOpen)
                .apiInfo(apiInfo())
                .groupName("EBA系统大屏")
                .select()
                .apis(RequestHandlerSelectors.basePackage("com.ebei.screen.controller.eba"))
                .paths(PathSelectors.any())
                .build();
    }

    @Bean(value = "energyPlatform")
    public Docket energyPlatform() {
        return new Docket(DocumentationType.SWAGGER_2)
                .enable(isOpen)
                .apiInfo(apiInfo())
                .groupName("能耗平台大屏")
                .select()
                .apis(RequestHandlerSelectors.basePackage("com.ebei.screen.controller.energy"))
                .paths(PathSelectors.any())
                .build();
    }

    private ApiInfo apiInfo() {
        return new ApiInfoBuilder()
                .title("南京金鹰数据大屏")
                .build();
    }

    @Override
    public void onApplicationEvent(WebServerInitializedEvent event) {
        try {
            InetAddress inetAddress = Inet4Address.getLocalHost();
            log.info("\033[1;33m项目启动成功！接口文档地址: http://" + inetAddress.getHostAddress() + ":" + event.getWebServer().getPort() + event.getApplicationContext().getEnvironment().getProperty("server.servlet.context-path") + "/doc.html\033[0m");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}