package com.ebei.screen.common.config.auth;

import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSONObject;
import com.ebei.screen.common.constants.CommonConstants;
import com.ebei.screen.common.util.LeviUtils;
import com.ebei.screen.service.EbaSystemService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * 登录认证拦截器
 *
 * @author Levi
 */
@Slf4j
@Component
public class LoginInterceptor implements HandlerInterceptor {

    @Value("${comm.mkf-service-url}")
    private String mkfServiceUrl;

    @Value("${comm.getProjectList}")
    private String getProjectList;

    @Autowired
    private EbaSystemService ebaSystemService;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) {
        log.info("===========begin 认证拦截器===========");
        if (request.getMethod().equals(HttpMethod.OPTIONS.name())) {
            response.setStatus(HttpServletResponse.SC_OK);
            return true;
        }
        String tokenValue = request.getHeader("token");
        // 判断token是否存在
        if (StringUtils.isBlank(tokenValue)) {
            LeviUtils.writeJson(response, "请登录");
            log.info("token是否存在:{}", tokenValue);
            return false;
        }
        String str = HttpRequest.get(mkfServiceUrl + tokenValue).execute().body();
        log.info("验证token结果：", str);
        JSONObject user = LeviUtils.getJsonFieldOne(str, "data", JSONObject.class);
        if (user == null || user.size() <= 0) {
            LeviUtils.writeJson(response, "请登录");
            log.info("token是否存在:{}", tokenValue);
            return false;
        }
        AuthUser authUser = AuthUser.builder().userId(Long.parseLong(user.getString("userId"))).userAccount(user.getString("userAccount")).userName(user.getString("userName")).build();
        LoginUtils.CURRENT_USER.set(authUser);
        ebaSystemService.getCrmProjectList(CommonConstants.EBA_ORDER);
        ebaSystemService.getCrmProjectList(CommonConstants.ZHIHUI_ORDER);
        ebaSystemService.getCrmProjectList(CommonConstants.WUYE_ORDER);
        ebaSystemService.getCrmProjectList(CommonConstants.PARK_ORDER);
        return true;
    }

}
