package com.ebei.screen.common.task;

import cn.hutool.core.date.DateUtil;
import com.ebei.screen.common.util.ParkUtils;
import com.ebei.screen.service.EbaSystemService;
import com.ebei.screen.service.EnergyPlatformService;
import com.ebei.screen.service.ParkSystemService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

/**
 * 大屏项目定时任务
 *
 * @author Levi.u
 * @date 2021/1/14 14:13
 */
@Slf4j
@Component
public class ScreenTask {

    @Autowired
    private EbaSystemService ebaSystemService;

    @Autowired
    private EnergyPlatformService energyPlatformService;

    @Autowired
    private ParkSystemService parkSystemService;

    /**
     * 每小时执行一次数据拉取
     */
    @Scheduled(cron = "0 0 */1 * * ?")
    public void execute() {
        try {
            log.info("定时开始执行 开始时间：{}", DateUtil.now());
            ebaSystemService.initData();
            energyPlatformService.initData();
            ParkUtils.getLoginToken();
            log.info("定时执行结束 结束时间：{}", DateUtil.now());
        } catch (Exception e) {
            log.info("定时任务执行出现错误：{}", e);
        }
    }

    /**
     * 每3秒处理6条数据
     */
    @Scheduled(cron = "*/3 * * * * ?")
    public void exec() {
        try {
            log.info("开始处理数据 开始时间：{}", DateUtil.now());
            // 获取10条进行处理
            boolean result = parkSystemService.taskExec(6);
            log.info("数据处理结束 结束时间：{} 处理结果：{}", DateUtil.now(), result);
        } catch (Exception e) {
            log.info("定时任务执行出现错误：{}", e);
        }
    }

}
