package com.ebei.screen.common.util;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.setting.dialect.Props;
import cn.hutool.setting.dialect.PropsUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.ebei.screen.common.constants.CommonConstants;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Component;

import java.text.NumberFormat;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * EBA大屏工具类
 *
 * @author Levi.u
 * @date 2021/1/13 21:03
 */
@Slf4j
@Component
public class EbaUtils {

    private static Integer SUCCESS_CODE = 200;
    private static MongoTemplate mongoTemplate;
    public static String token;
    public static JSONObject project;
    public static JSONObject deviceSystem;
    public static JSONObject deviceType;
    public static JSONObject deviceParam;
    public static JSONObject deviceAlarmType;
    public static JSONObject fireDeviceType;
    public static JSONObject realDataDevice;
    public static JSONObject crmProjectInfo;

    @Autowired
    public void setMongoTemplate(MongoTemplate mongoTemplate) {
        EbaUtils.mongoTemplate = mongoTemplate;
    }

    /**
     * 判断执行结果是否成功
     *
     * @param result true成功 false失败
     * @return
     */
    public static boolean isSuccess(String result) {
        return SUCCESS_CODE.equals(JSON.parseObject(result).getInteger("code"));
    }

    public static List<Map> getEmptyTemplate() {
        // 初始化模板
        List<Map> monthList = new ArrayList<>();
        Integer monthDays = LeviUtils.getMonthDays(LocalDateTime.now().getMonthValue());
        for (int b = 1; b <= monthDays; b++) {
            List<Map> dayList = new ArrayList<>();
            for (int c = 0; c <= 23; c++) {
                dayList.add(Levi.by("hour", c).set("data", new ArrayList<>()));
            }
            monthList.add(Levi.by("day", b).set("data", dayList));
        }
        return monthList;
    }

    /**
     * 用于存储数据(不带统计数字)
     *
     * @param obj            插入的对象
     * @param pkId           区分键
     * @param collectionName 集合名字
     * @param flag           true 覆盖原有数据只保留一条最新的 false追加原有数据
     * @param mongoId        如果设置了该参数 则取该参数id
     * @return
     */
    public static List<Map> setDataField(Object obj, String pkId, String collectionName, boolean flag, String mongoId) {
        LocalDateTime now = LocalDateTime.now();
        int year = now.getYear();
        int month = now.getMonthValue();
        int day = now.getDayOfMonth();
        int hour = now.getHour();
        Map monthMap = mongoTemplate.findById(StringUtils.isNotBlank(mongoId) ? mongoId : (pkId + "_" + year + "_" + month), Map.class, collectionName);
        if (MapUtil.isEmpty(monthMap)) {
            monthMap = Levi.by("_id", StringUtils.isNotBlank(mongoId) ? mongoId : (pkId + "_" + year + "_" + month)).set("data", getEmptyTemplate());
        }
        List<Map> dayList = (List<Map>) monthMap.get("data");
        Map dayMap = dayList.get(day - 1);
        List<Map> hourList = (List<Map>) dayMap.get("data");
        Map hourMap = hourList.get(hour);
        List<Map> dataList = (List<Map>) hourMap.get("data");
        if (CollUtil.isNotEmpty(dataList) && !flag) {
            dataList.addAll((List<Map>) obj);
            hourMap.put("data", dataList);
        } else {
            hourMap.put("data", obj);
        }
        return dayList;
    }

    /**
     * 用于存储数据(带统计数字)
     *
     * @param key            count字段key名称
     * @param obj            插入的对象
     * @param pkId           区分键
     * @param collectionName 集合名字
     * @param flag           true 覆盖原有数据只保留一条最新的 false追加原有数据
     * @return
     */
    public static Map setDataFieldCount(String key, Object obj, String pkId, String collectionName, boolean flag) {
        LocalDateTime now = LocalDateTime.now();
        int year = now.getYear();
        int month = now.getMonthValue();
        int day = now.getDayOfMonth();
        int hour = now.getHour();
        List<Map> objList = (List<Map>) obj;
        Map monthMap = mongoTemplate.findById(pkId + "_" + year + "_" + month, Map.class, collectionName);
        if (MapUtil.isEmpty(monthMap)) {
            monthMap = Levi.by("_id", pkId + "_" + year + "_" + month).set("data", getEmptyTemplate());
        }
        List<Map> dayList = (List<Map>) monthMap.get("data");
        Map dayMap = dayList.get(day - 1);
        List<Map> hourList = (List<Map>) dayMap.get("data");
        Map hourMap = hourList.get(hour);
        List<Map> dataList = (List<Map>) hourMap.get("data");
        if (CollUtil.isNotEmpty(dataList) && !flag) {
            dataList.addAll(objList);
            hourMap.put("data", dataList);
            int size = objList.size();
            for (Map x : dataList) {
                String projectId = x.get("ProjectId").toString();
                hourMap.put(key + "_" + projectId, addCount(hourMap, size, key + "_" + projectId));
                dayMap.put(key + "_" + projectId, addCount(dayMap, size, key + "_" + projectId));
                monthMap.put(key + "_" + projectId, addCount(monthMap, size, key + "_" + projectId));
            }
            hourMap.put(key, addCount(hourMap, size, key));
            dayMap.put(key, addCount(dayMap, size, key));
            monthMap.put(key, addCount(monthMap, size, key));
        } else {
            hourMap.put("data", obj);
            int size = objList.size();
            for (Map x : objList) {
                String projectId = x.get("ProjectId").toString();
                hourMap.put(key + "_" + projectId, addCount(hourMap, size, key + "_" + projectId) - dataList.size());
                dayMap.put(key + "_" + projectId, addCount(dayMap, size, key + "_" + projectId) - dataList.size());
                monthMap.put(key + "_" + projectId, addCount(monthMap, size, key + "_" + projectId) - dataList.size());
            }
            hourMap.put(key, addCount(hourMap, size, key) - dataList.size());
            dayMap.put(key, addCount(dayMap, size, key) - dataList.size());
            monthMap.put(key, addCount(monthMap, size, key) - dataList.size());
        }
        return monthMap;
    }

    private static Integer addCount(Map a, Integer b, String key) {
        Integer count = (Integer) a.get(key);
        return count == null ? b : (count + b);
    }

    /**
     * 获取同比数据
     *
     * @param old 去年数
     * @param now 今年数
     * @return
     */
    public static String getTongBi(Integer old, Integer now) {
        float oldF = (float) old;
        float nowF = (float) now;
        if (old == null || old == 0) {
            return "100%";
        }
        NumberFormat format = NumberFormat.getPercentInstance();
        float result = (nowF - oldF) / oldF;
        return format.format(result);
    }

    /**
     * 根据ebei项目id获取关联的金鹰项目id
     *
     * @param projectId
     * @return
     */
    public static String getProjectId(String projectId) {
        JSONObject obj = mongoTemplate.findById("relevanceProject", JSONObject.class, "relevanceProject");
        JSONObject data = obj != null ? obj.getJSONObject("data") : new JSONObject();
        return data.getString(projectId);
    }

    /**
     * 获取项目ids
     *
     * @return
     */
    public static Set<String> getProjectIds() {
        Set<String> sortSet = new TreeSet<>(Comparator.naturalOrder());
        sortSet.addAll(project.keySet());
        return sortSet;
    }

    /**
     * 给eba项目排序
     *
     * @param list
     * @return
     */
    public static List<Map> sortList(List<Map> list) {
        JSONObject obj = mongoTemplate.findById(CommonConstants.EBA_ORDER, JSONObject.class, CommonConstants.EBA_ORDER);
        List<String> crmIds = obj != null ? obj.getJSONArray("data").toJavaList(String.class) : new JSONArray().toJavaList(String.class);
        Collections.sort(list, Comparator.comparingInt(o -> crmIds.indexOf(o.get("projectId").toString())));
        return list.stream().filter(x -> x.get("projectName") != null).collect(Collectors.toList());
    }

    /**
     * 根据key恢复项目默认排序
     *
     * @return
     */
    public static List<String> resetSortList(String key) {
        Props prop = PropsUtil.get(key);
        List<String> objects = prop.keySet().stream().map(x -> (String) x).collect(Collectors.toList());
        Collections.sort(objects, Comparator.comparingInt(o -> Integer.parseInt(prop.get(o).toString())));
        Query query = new Query();
        query.addCriteria(Criteria.where("_id").is(key + CommonConstants.ORDER_PREFIX));
        Update update = new Update();
        update.set("data", objects);
        mongoTemplate.upsert(query, update, key + CommonConstants.ORDER_PREFIX);
        return objects;
    }

    /**
     * 保持原有顺序 依据接口数据更新crm列表顺序(自定义传序号顺序来源)
     *
     * @param projectLists 当前crm项目接口返回集合
     * @param key          当前序号顺序来源mongo的key
     * @return
     */
    public static List<String> getNowCrmProjectSort(List<Map> projectLists, String key) {
        JSONObject obj = mongoTemplate.findById(key, JSONObject.class, key);
        List<String> list1 = obj != null ? obj.getJSONArray("data").toJavaList(String.class) : new JSONArray().toJavaList(String.class);
        List<String> list2 = projectLists.stream().map(x -> x.get("projectId").toString()).collect(Collectors.toList());
        List<String> collect = CollUtil.disjunction(CollUtil.intersection(list1, list2), list2).stream().collect(Collectors.toList());
        List<String> subtract = CollUtil.disjunction(CollUtil.disjunction(CollUtil.intersection(list1, list2), list2), CollUtil.disjunction(list1, list2)).stream().collect(Collectors.toList());
        list1.removeAll(subtract);
        list1.addAll(collect);
        Query query = new Query();
        query.addCriteria(Criteria.where("_id").is(key));
        Update update = new Update();
        update.set("data", list1);
        mongoTemplate.upsert(query, update, key);
        return list1;
    }
}
