package com.ebei.screen.common.util;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ebei.screen.common.constants.DmonDeviceCode;
import com.ebei.screen.common.constants.EnergyPlatformConstants;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.text.MessageFormat;
import java.text.NumberFormat;
import java.time.LocalDateTime;
import java.util.*;

/**
 * 能耗平台大屏工具类
 *
 * @author Levi.u
 * @date 2021/1/15 11:35
 */
@Slf4j
@Component
public class EnergyUtils {

    private static MongoTemplate mongoTemplate;
    public static String token;
    public static String refreshToken;
    public static List<String> boxIds;
    public static Map<String, List<String>> groupIds;
    public static Map<String, List<String>> dmonIds;


    @Autowired
    public void setMongoTemplate(MongoTemplate mongoTemplate) {
        EnergyUtils.mongoTemplate = mongoTemplate;
    }


    public static List<Map> getEmptyTemplate() {
        // 初始化模板
        List<Map> monthList = new ArrayList<>();
        //Integer monthDays = LeviUtils.getMonthDays(LocalDateTime.now().getMonthValue());
        Integer monthDays = 31;
        for (int b = 1; b <= monthDays; b++) {
            List<Map> dayList = new ArrayList<>();
            for (int c = 0; c <= 23; c++) {
                dayList.add(Levi.by("hour", c).set("data", new ArrayList<>()));
            }
            monthList.add(Levi.by("day", b).set("data", dayList));
        }
        return monthList;
    }

    /**
     * 获取拼接后的url地址
     *
     * @param url  原地址
     * @param objs 按照顺序输入任意多个参数
     * @return
     */
    public static String getUrl(String url, Object... objs) {
        return MessageFormat.format(url, objs);
    }

    /**
     * 获取登录token参数
     *
     * @return
     */
    public static List<String> loginToken() {
        return Arrays.asList("username=" + EnergyPlatformConstants.username, "password=" + EnergyPlatformConstants.password, "scope=" + EnergyPlatformConstants.scope, "grant_type=" + EnergyPlatformConstants.grant_type, "client_id=" + EnergyPlatformConstants.client_id, "client_secret=" + EnergyPlatformConstants.client_secret);
    }

    /**
     * 获取刷新token参数
     *
     * @return
     */
    public static List<String> refreshToken() {
        return Arrays.asList("refresh_token=" + EnergyUtils.refreshToken, "scope=" + EnergyPlatformConstants.scope, "grant_type=" + EnergyPlatformConstants.grant_type_two, "client_id=" + EnergyPlatformConstants.client_id, "client_secret=" + EnergyPlatformConstants.client_secret);
    }

    /**
     * 获取token
     *
     * @param bodys
     * @return
     */
    public static String doToken(List<String> bodys) {
        String result = HttpRequest.post(EnergyPlatformConstants.coreConnectToken)
                .header("content-type", "application/x-www-form-urlencoded")
                .body(String.join("&", bodys))
                .execute().body();
        JSONObject obj = JSON.parseObject(result);
        if (obj.getString("error") != null) {
            return null;
        }
        String token = obj.getString("token_type") + " " + obj.getString("access_token");
        String refreshToken = obj.getString("refresh_token");
        Query query = new Query();
        query.addCriteria(Criteria.where("_id").is("token"));
        Update update = new Update();
        update.set("data", token);
        update.set("expire", LocalDateTime.now());
        mongoTemplate.upsert(query, update, "energyAuth");
        EnergyUtils.token = token;
        EnergyUtils.refreshToken = refreshToken;
        return token;
    }

    /**
     * 用于存储数据并计算用电量
     *
     * @param obj            插入的对象
     * @param pkId           区分键
     * @param collectionName 集合名字
     * @return
     */
    public static Map setDataField(Object obj, String pkId, String collectionName) {
        LocalDateTime now = LocalDateTime.now();
        int year = now.getYear();
        int month = now.getMonthValue();
        int day = now.getDayOfMonth();
        int hour = now.getHour();
        List<Map> objList = (List<Map>) obj;
        Map monthMap = mongoTemplate.findById(pkId + "_" + year + "_" + month, Map.class, collectionName);
        if (MapUtil.isEmpty(monthMap)) {
            monthMap = Levi.by("_id", pkId + "_" + year + "_" + month).set("data", getEmptyTemplate());
        }
        List<Map> dayList = (List<Map>) monthMap.get("data");
        Map dayMap = dayList.get(day - 1);
        List<Map> hourList = (List<Map>) dayMap.get("data");
        Map hourMap = hourList.get(hour);
        List<Map> dataList = (List<Map>) hourMap.get("data");
        // 获取获取本次即将存入数据值
        Map<String, Double> thisValue = variousData(objList);
        // 尝试获取是否存在历史的数据
        Map<String, Double> deleteValue = variousData(dataList);
        // 存放上一次数据值 当前值减上次即为本次的值
        List<Map> lastDataList;
        // 如果当前时间为0点 取上一天23小时数据进行比较
        if (hour == 0) {
            Map lastDayMap = dayList.get(day - 2);
            List<Map> lastHourList = (List<Map>) lastDayMap.get("data");
            Map lastHourMap = lastHourList.get(23);
            lastDataList = (List<Map>) lastHourMap.get("data");
        } else {
            Map lastHourMap = hourList.get(hour - 1);
            lastDataList = (List<Map>) lastHourMap.get("data");
        }
        // 如果上一次是空的 则默认本次为开头第一个
        if (CollUtil.isEmpty(lastDataList)) {
            lastDataList = LeviUtils.deepCopy(objList);
        }
        // 尝试获取上一小时的数据
        Map<String, Double> lastValue = variousData(lastDataList);
        // 获取本小时的具体数值
        Map<String, Double> nowValue = nowValue(lastValue, thisValue);
        hourMap.put("data", obj);
        for (int i = 1; i <= 6; i++) {
            hourMap.put("zj" + i, nowValue.get("zj" + i));
            dayMap.put("zj" + i, addCount(dayMap, nowValue, deleteValue, "zj" + i));
            monthMap.put("zj" + i, addCount(monthMap, nowValue, deleteValue, "zj" + i));
        }
        hourMap.put("all", nowValue.get("all"));
        dayMap.put("all", addCount(dayMap, nowValue, deleteValue, "all"));
        monthMap.put("all", addCount(monthMap, nowValue, deleteValue, "all"));
        return monthMap;
    }

    private static Double addCount(Map a, Map<String, Double> nowValue, Map<String, Double> deleteValue, String key) {
        Double b = nowValue.get(key);
        Double count = Double.valueOf(a.get(key) == null ? "0" : a.get(key).toString());
        double result = count == null ? b : Double.sum(count, b);
        return result - deleteValue.get(key);
    }

    /**
     * 计算当前的值
     *
     * @param lastValue 上一小时的值
     * @param thisValue 当前要插的值
     * @return
     */
    private static Map<String, Double> nowValue(Map<String, Double> lastValue, Map<String, Double> thisValue) {
        Map<String, Double> map = new HashMap(16);
        for (int i = 1; i <= 6; i++) {
            map.put("zj" + i, thisValue.get("zj" + i) - lastValue.get("zj" + i));
        }
        map.put("all", thisValue.get("all") - lastValue.get("all"));
        return map;
    }

    /**
     * 获取每个机箱电数以及总电数
     *
     * @param dataList
     * @return
     */
    private static Map<String, Double> variousData(List<Map> dataList) {
        boolean flag = CollUtil.isEmpty(dataList);
        JSONObject data = new JSONObject();
        if (!flag) {
            dataList.forEach(x -> data.put(x.get("id").toString(), x.get("value").toString()));
        }
        double zj1 = flag ? 0.0 : data.getDoubleValue(DmonDeviceCode.ONE.getCode());
        double zj2 = flag ? 0.0 : data.getDoubleValue(DmonDeviceCode.TWO.getCode());
        double zj3 = flag ? 0.0 : data.getDoubleValue(DmonDeviceCode.THREE.getCode());
        double zj4 = flag ? 0.0 : data.getDoubleValue(DmonDeviceCode.FOUR.getCode());
        double zj5 = flag ? 0.0 : data.getDoubleValue(DmonDeviceCode.FIVE.getCode());
        double zj6 = flag ? 0.0 : data.getDoubleValue(DmonDeviceCode.SIX.getCode());
        double all = flag ? 0.0 : data.getDoubleValue(DmonDeviceCode.ALL.getCode());
        return Levi.by("zj1", zj1).set("zj2", zj2).set("zj3", zj3).set("zj4", zj4).set("zj5", zj5).set("zj6", zj6).set("all", all);
    }

    public static String getPercent(Double one, Double two) {
        if (one == 0) {
            return "0%";
        }
        if (two == 0) {
            return "100%";
        }
        NumberFormat format = NumberFormat.getPercentInstance();
        Double result = one / two;
        return format.format(result);
    }

    public static String divide(double a, double b, int scale) {
        if (a == 0 || b == 0) {
            return "0.00";
        }
        BigDecimal bd1 = new BigDecimal(Double.toString(a));
        BigDecimal bd2 = new BigDecimal(Double.toString(b));
        return bd1.divide(bd2, scale, BigDecimal.ROUND_HALF_UP).toString();
    }

    /**
     * 获取具体某一天的某一项统计数据
     *
     * @param key 要查询数据的key
     * @param map 要查询的map(月份级别)
     * @param day 查询哪一天
     * @return
     */
    public static Double getDayInfo(String key, Map map, Integer day) {
        if (MapUtil.isNotEmpty(map)) {
            Map obj = ((List<Map>) map.get("data")).get(day - 1);
            if (MapUtil.isNotEmpty(obj)) {
                return Double.valueOf((obj.get(key) == null ? "0" : obj.get(key).toString()));
            }
            return 0.0;
        }
        return 0.0;
    }

    public static List<String> addSameList(List<String> list, String s) {
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>(Arrays.asList(s));
        }
        list.add(s);
        return list;
    }

}
