package com.ebei.screen.common.util;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.*;

/**
 * 自用工具类
 *
 * @author Levi
 */
public class Levi<K, V> extends LinkedHashMap<K, V> {

    public Levi() {
    }

    public static <K, V> Levi by(K key, V value) {
        return new Levi<K, V>().set(key, value);
    }

    public static <K, V> Levi create() {
        return new Levi<K, V>();
    }

    public Levi<K, V> set(K key, V value) {
        super.put(key, value);
        return this;
    }

    public Levi<K, V> set(Map<K, V> map) {
        super.putAll(map);
        return this;
    }

    public Levi<K, V> set(Levi<K, V> levi) {
        super.putAll(levi);
        return this;
    }

    public Levi<K, V> delete(K key) {
        super.remove(key);
        return this;
    }

    public Levi<K, V> delete(K... keys) {
        Arrays.asList(keys).stream().forEach(x -> super.remove(x));
        return this;
    }

    public List<V> get(K... keys) {
        List<V> list = new ArrayList<>();
        Arrays.asList(keys).stream().forEach(x -> list.add(super.get(x)));
        list.removeAll(Collections.singleton(null));
        return list;
    }

    public String getStr(Object key) {
        Object s = get(key);
        return s != null ? s.toString() : null;
    }

    public Integer getInt(Object key) {
        Number n = (Number) get(key);
        return n != null ? n.intValue() : null;
    }

    public BigDecimal getBigDecimal(Object key) {
        Object value = get(key);
        if (value == null) {
            return null;
        } else if (value instanceof BigDecimal) {
            return (BigDecimal) value;
        } else if (value instanceof BigInteger) {
            return new BigDecimal((BigInteger) value);
        } else {
            String strVal = value.toString();
            return strVal.length() == 0 ? null : new BigDecimal(strVal);
        }
    }

    public Long getLong(Object key) {
        Number n = (Number) get(key);
        return n != null ? n.longValue() : null;
    }

    public Number getNumber(Object key) {
        return (Number) get(key);
    }

    public Boolean getBoolean(Object key) {
        Object v = get(key);
        return null == v ? false : Boolean.valueOf(v.toString());
    }

    @Override
    public boolean equals(Object levi) {
        return levi instanceof Levi && super.equals(levi);
    }
}
