package com.ebei.screen.common.util;

import lombok.extern.slf4j.Slf4j;

import java.util.concurrent.*;
import java.util.function.Supplier;

@Slf4j
public class RunnableUtils {
    private static final int IN_USE_SIZE = 40;
    private static final ExecutorService service = createPoolExecutor();

    public static void start(Running run) {
        try {
            service.execute(() -> run.run());
        } catch (Exception e) {
            log.error("RunnableUtils==>start:", e);
        }
    }

    public static <T> T submit(Supplier<T> call) {
        try {
            Future<T> future = service.submit(() -> call.get());
            return future.get();
        } catch (Exception e) {
            log.error("RunnableUtils==>submit:", e);
        }
        return null;
    }

    public static void stop() {
        try {
            service.shutdownNow();
        } catch (Exception e) {
            log.error("RunnableUtils==>stop:", e);
        }
    }

    private static ExecutorService createPoolExecutor() {
        return new ThreadPoolExecutor(IN_USE_SIZE, IN_USE_SIZE, 60L, TimeUnit.SECONDS, new SynchronousQueue());
    }

}
