package com.ebei.screen.common.util;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.concurrent.BasicThreadFactory;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.concurrent.*;

/**
 * 延迟执行工具类
 *
 * @author Levi.u
 * @date 2021/2/23 11:14
 */
@Slf4j
@Component
public class ScheduledUtils {

    private static final int IN_USE_SIZE = Runtime.getRuntime().availableProcessors() + 1;
    private static ScheduledExecutorService scheduleService = new ScheduledThreadPoolExecutor(IN_USE_SIZE,
            new BasicThreadFactory.Builder().namingPattern("schedule-pool-%d").daemon(true).build());

    public static void start(Callable run, Long time) {
        try {
            ScheduledFuture<?> schedule = scheduleService.schedule(() -> run.call(), time, TimeUnit.SECONDS);
            if (schedule != null) {
                List<String> list = (List<String>) schedule.get();
                list.forEach(System.out::println);
            }
        } catch (Exception e) {
            log.error("Scheduled==>start:", e);
        }
    }

    public static void stop() {
        scheduleService.shutdown();
    }
}
