package com.ebei.screen.controller.eba;

import com.ebei.screen.common.constants.CommonConstants;
import com.ebei.screen.common.response.ResponseBean;
import com.ebei.screen.common.util.EbaUtils;
import com.ebei.screen.modules.req.QueryPageReq;
import com.ebei.screen.modules.req.RealTimeReq;
import com.ebei.screen.service.EbaSystemService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * @author Liu.x
 * @description EBA系统大屏
 */
@Api(tags = "EBA系统大屏")
@RestController
@RequestMapping("eba/system")
public class EbaSystemController {

    @Autowired
    private EbaSystemService ebaSystemService;

    // @ApiOperation("实时数据回调接口")
    @PostMapping(value = "/realTimeCallback", produces = "application/json; charset=utf-8")
    public Integer realTimeCallback(@RequestBody Map params) {
        return ebaSystemService.realTimeCallback(params);
    }

    @ApiOperation("获取项目列表 ✔")
    @PostMapping("/getProjectList")
    public ResponseBean getProjectList() {
        return ebaSystemService.getProjectList();
    }

    @ApiOperation("获取实时设备列表 ✔")
    @PostMapping(value = {"/getDeviceList/{projectId}", "/getDeviceList"})
    public ResponseBean getDeviceList(@PathVariable(required = false) String projectId) {
        return ebaSystemService.getDeviceList(projectId);
    }

    @ApiOperation("获取实时火警和预警 ✔")
    @PostMapping(value = {"/getRealTimeList/{projectId}", "/getRealTimeList"})
    public ResponseBean getRealTimeList(@PathVariable(required = false) String projectId) {
        projectId = EbaUtils.getProjectId(projectId);
        return ebaSystemService.getRealTimeList(projectId);
    }

    @ApiOperation("获取crm项目列表 ✔")
    @PostMapping("/getCrmProjectList")
    public ResponseBean getCrmProjectList() {
        return ebaSystemService.getCrmProjectList(CommonConstants.EBA_ORDER);
    }

    @ApiOperation("保存项目关联关系 ✔")
    @PostMapping("/relevanceProject")
    public ResponseBean relevanceProject(@RequestBody Map<String, String> params) {
        return ebaSystemService.relevanceProject(params);
    }

    @ApiOperation("查询已配置的crm项目列表 ✔")
    @PostMapping("/getShowProjectList")
    public ResponseBean getShowProjectList() {
        return ebaSystemService.getShowProjectList();
    }

    @ApiOperation("分页查询所有已配置关联的项目列表 ✔")
    @PostMapping("/getProjectListByPage")
    public ResponseBean getProjectListByPage(@RequestBody QueryPageReq pageBean) {
        return ebaSystemService.getProjectListByPage(pageBean);
    }

    @ApiOperation("删除已有的关联项目 ✔")
    @PostMapping("/deleteRelevance/{ebeiProjectId}")
    public ResponseBean deleteRelevance(@PathVariable String ebeiProjectId) {
        return ebaSystemService.deleteRelevance(ebeiProjectId);
    }

    @ApiOperation("分页查询火警和故障信息 ✔")
    @PostMapping("/getRealTimeListByPage")
    public ResponseBean getRealTimeListByPage(@RequestBody RealTimeReq params) {
        params.setProjectId(EbaUtils.getProjectId(params.getProjectId()));
        return ebaSystemService.getRealTimeListByPage(params);
    }

    @ApiOperation("根据文档名称修复火警故障数字")
    @PostMapping("/fixDate/{name}")
    public ResponseBean fixDate(@PathVariable String name) {
        return ebaSystemService.fixDate(name);
    }

    @ApiOperation("根据文档名称修复丢失天数")
    @PostMapping("/fixDay/{name}")
    public ResponseBean fix(@PathVariable String name) {
        return ebaSystemService.fix(name);
    }


    @ApiOperation("清除某个项目所有实时火警历史数据")
    @PostMapping("/delRealTimeByName/{name}")
    public ResponseBean delRealTimeByName(@PathVariable String name) {
        return ebaSystemService.delRealTimeByName(name);
    }

    @ApiOperation("恢复默认排序")
    @PostMapping("/resetSort/{type}")
    public ResponseBean resetSort(@PathVariable String type) {
        return ebaSystemService.resetSort(type);
    }

}
