package com.ebei.screen.controller.park;

import cn.hutool.core.lang.UUID;
import com.ebei.screen.common.constants.CommonConstants;
import com.ebei.screen.common.response.ResponseBean;
import com.ebei.screen.common.util.Levi;
import com.ebei.screen.modules.req.*;
import com.ebei.screen.service.EbaSystemService;
import com.ebei.screen.service.ParkSystemService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Map;

/**
 * @author Liu.x
 * @description 车场系统大屏
 */
@Api(tags = "车场系统大屏")
@RestController
@RequestMapping("park")
@Slf4j
public class ParkSystemController {

    @Autowired
    private ParkSystemService parkSystemService;

    @Autowired
    private EbaSystemService ebaSystemService;

    @ApiOperation("车场信息数据推送")
    @PostMapping("/parkInfo")
    public Levi parkInfo(@RequestBody Map params) {
        return parkSystemService.exec("parkInfo" + UUID.fastUUID(), params, true);
    }

    @ApiOperation("车位信息数据推送")
    @PostMapping("/parkSpace")
    public Levi parkSpace(@RequestBody Map params) {
        return parkSystemService.exec("parkSpace" + UUID.fastUUID(), params, true);
    }

    @ApiOperation("入场信息数据推送")
    @PostMapping("/parkIn")
    public Levi parkIn(@RequestBody Map params) {
        return parkSystemService.exec("parkIn" + UUID.fastUUID(), params, true);
    }

    @ApiOperation("出场信息数据推送")
    @PostMapping("/parkOut")
    public Levi parkOut(@RequestBody Map params) {
        return parkSystemService.exec("parkOut" + UUID.fastUUID(), params, true);
    }

    @ApiOperation("订单信息数据推送")
    @PostMapping("/parkOrder")
    public Levi parkOrder(@RequestBody Map params) {
        return parkSystemService.exec("parkOrder" + UUID.fastUUID(), params, true);
    }

    @ApiOperation("收费记录数据推送")
    @PostMapping("/parkCharge")
    public Levi parkCharge(@RequestBody Map params) {
        return parkSystemService.exec("parkCharge" + UUID.fastUUID(), params, false);
    }

    @ApiOperation("服务信息数据推送")
    @PostMapping("/parkServiceInfo")
    public Levi parkServiceInfo(@RequestBody Map params) {
        return parkSystemService.exec("parkServiceInfo" + UUID.fastUUID(), params, true);
    }

    @ApiOperation("今日车场流量分析查询 ✔")
    @PostMapping(value = {"/getTodayFlowAnalysis/{parkCode}", "/getTodayFlowAnalysis"})
    public ResponseBean getTodayFlowAnalysis(@PathVariable(required = false) String parkCode) {
        return parkSystemService.getTodayFlowAnalysis(parkCode);
    }

    @ApiOperation("今日车场流量分析分页查询详情 ✔")
    @PostMapping("/getTodayFlowAnalysisByPage")
    public ResponseBean getTodayFlowAnalysisByPage(@RequestBody FlowAnalysisReq params) {
        return parkSystemService.getTodayFlowAnalysisByPage(params);
    }

    @ApiOperation("今日车场流量分析分页查询折线 ✔")
    @PostMapping("/getTodayFlowAnalysisByLine")
    public ResponseBean getTodayFlowAnalysisByLine(@RequestBody FlowAnalysisReq params) {
        return parkSystemService.getTodayFlowAnalysisByLine(params);
    }

    @ApiOperation("入场车辆通车时间段统计查询 ✔")
    @PostMapping("/inCarAnalysis")
    public ResponseBean inCarAnalysis(@RequestBody InCarAnalysisReq params) {
        return parkSystemService.inCarAnalysis(params);
    }

    @ApiOperation("入场车辆通车时间段统计分页查询详情 ✔")
    @PostMapping("/inCarAnalysisByPage")
    public ResponseBean inCarAnalysisByPage(@RequestBody InCarAnalysisPageReq params) {
        return parkSystemService.inCarAnalysisByPage(params);
    }

    @ApiOperation("今日停车时长统计查询 ✔")
    @PostMapping(value = {"/parkingHoursToday/{parkCode}", "/parkingHoursToday"})
    public ResponseBean parkingHoursToday(@PathVariable(required = false) String parkCode) {
        return parkSystemService.parkingHoursToday(parkCode);
    }

    @ApiOperation("今日停车时长统计分页查询详情 ✔")
    @PostMapping("/parkingHoursTodayByPage")
    public ResponseBean parkingHoursTodayByPage(@RequestBody ParkHoursPageReq params) {
        return parkSystemService.parkingHoursTodayByPage(params);
    }

    @ApiOperation("关键指标查询 ✔")
    @PostMapping(value = {"/keyIndex/{parkCode}", "/keyIndex"})
    public ResponseBean keyIndex(@PathVariable(required = false) String parkCode) {
        return parkSystemService.keyIndex(parkCode);
    }

    @ApiOperation("关键指标分页查询详情 ✔")
    @PostMapping("/keyIndexByPage")
    public ResponseBean keyIndexByPage(@RequestBody KeyIndexPageReq params) {
        return parkSystemService.keyIndexByPage(params);
    }

    @ApiOperation("不同收费方式停车统计查询 ✔")
    @PostMapping(value = {"/differentPay/{parkCode}", "/differentPay"})
    public ResponseBean differentPay(@PathVariable(required = false) String parkCode) {
        return parkSystemService.differentPay(parkCode);
    }

    @ApiOperation("不同收费方式停车统计分页查询详情 ✔")
    @PostMapping("/differentPayByPage")
    public ResponseBean differentPayByPage(@RequestBody DifferentPayPageReq params) {
        return parkSystemService.differentPayByPage(params);
    }

    @ApiOperation("获取crm项目列表 ✔")
    @PostMapping("/getCrmProjectList")
    public ResponseBean getCrmProjectList() {
        return ebaSystemService.getCrmProjectList(CommonConstants.PARK_ORDER);
    }

    @ApiOperation("查询已配置的crm项目列表 ✔")
    @PostMapping("/getShowProjectList")
    public ResponseBean getShowProjectList() {
        return parkSystemService.getShowProjectList();
    }

    @ApiOperation("删除已有的关联项目 ✔")
    @PostMapping("/deleteRelevance/{ebeiProjectId}")
    public ResponseBean deleteRelevance(@PathVariable String ebeiProjectId) {
        return parkSystemService.deleteRelevance(ebeiProjectId);
    }

    @ApiOperation("获取项目列表 ✔")
    @PostMapping("/getProjectList")
    public ResponseBean getProjectList() {
        return parkSystemService.getProjectList();
    }

    @ApiOperation("异常趋势统计查询 ✔")
    @PostMapping(value = {"/exceptionAnalysis/{parkCode}", "/exceptionAnalysis"})
    public ResponseBean exceptionAnalysis(@PathVariable(required = false) String parkCode) {
        return parkSystemService.exceptionAnalysis(parkCode);
    }

    @ApiOperation("异常趋势统计分页查询详情 ✔")
    @PostMapping("/exceptionAnalysisByPage")
    public ResponseBean exceptionAnalysisByPage(@RequestBody ExceptionTrendReq params) {
        return parkSystemService.exceptionAnalysisByPage(params);
    }

    @ApiOperation("根据集合名称修复所有指定日期后的数字")
    @PostMapping("/fixDate/{collection}/{date}")
    public ResponseBean fixDate(@PathVariable String collection, @PathVariable String date) {
        return parkSystemService.fixDate(collection, date);
    }

    @ApiOperation(value = "异常信息导出")
    @PostMapping("/exportException")
    public void exportException(@RequestBody ExceptionTrendReq params, HttpServletResponse response) {
        parkSystemService.exportException(params, response);
    }

}
