package com.ebei.screen.controller.video;

import cn.hutool.core.bean.BeanUtil;
import com.ebei.screen.common.response.ResponseBean;
import com.ebei.screen.modules.po.CameraDevice;
import com.ebei.screen.modules.po.MachineRoom;
import com.ebei.screen.modules.po.Project;
import com.ebei.screen.modules.po.VideoScreenListVo;
import com.ebei.screen.service.CameraDeviceService;
import com.ebei.screen.service.MachineRoomService;
import com.ebei.screen.service.ProjectService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;

/**
 * @author Liu.x
 * @description 视频监控大屏
 */

@Slf4j
@Validated
@RestController
@RequestMapping("/video")
@AllArgsConstructor
@Api(value = "视频监控", tags = "视频监控api")
public class VideoMonitorController {

    @Autowired
    private ProjectService projectService;
    @Autowired
    private CameraDeviceService cameraDeviceService;
    @Autowired
    private MachineRoomService machineRoomService;

    @ApiOperation("保存项目")
    @PostMapping("/saveProject")
    public ResponseBean saveProject(@RequestBody Project project) {
        projectService.saveProject(project);
        return ResponseBean.success();
    }

    @ApiOperation("修改项目")
    @PostMapping("/updateProject")
    public ResponseBean updateProject(@RequestBody Project project) {
        if (BeanUtil.isEmpty(project.getId())) {
            return ResponseBean.fail("id不能为空");
        }
        return projectService.updateProject(project);
    }

    @ApiOperation("分页查询项目")
    @PostMapping("/queryProject/{page}/{pageSize}")
    public ResponseBean queryProjectPage(@PathVariable("page") Integer page,
                                         @PathVariable("pageSize") Integer pageSize,
                                         @RequestBody Project project) {

        return projectService.queryProjectPage(page - 1, pageSize, project.getName());
    }

    @ApiOperation("同步萤石设备")
    @GetMapping("/syncAllYsDevice")
    public ResponseBean syncAllYsDevice() {

        try {
            cameraDeviceService.syncAllYsDevice();
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        return ResponseBean.success("后台同步中，请不要重复点击!");

    }

    @ApiOperation("分页查询萤石设备")
    @PostMapping("/queryYsDevice/{page}/{pageSize}")
    public ResponseBean queryYsDevice(@PathVariable("page") @NotNull(message = "URL参数：当前页的页码不能为空") Integer page,
                                      @PathVariable("pageSize") @NotNull(message = "URL参数：当前页的页数不能为空") Integer pageSize,
                                      @RequestBody CameraDevice cameraDevice) {

        return cameraDeviceService.queryProjectPage(page - 1, pageSize, cameraDevice.getDeviceName(), cameraDevice.getDeviceSerial(), cameraDevice.getStatus());
    }

    @ApiOperation("保存机房")
    @PostMapping("/saveMachineRoom")
    public ResponseBean saveMachineRoom(@RequestBody MachineRoom machineRoom) {
        machineRoomService.saveMachineRoom(machineRoom);
        return ResponseBean.success();
    }

    @ApiOperation("修改机房(可用于关联监控设备)")
    @PostMapping("/updateMachineRoom")
    public ResponseBean updateMachineRoom(@RequestBody MachineRoom machineRoom) {
        if (BeanUtil.isEmpty(machineRoom.getId())) {
            return ResponseBean.fail("id不能为空");
        }
        machineRoomService.updateMachineRoom(machineRoom);
        return ResponseBean.success();
    }

    @ApiOperation("根据项目分页查询机房")
    @PostMapping("/queryMachineRoom/{page}/{pageSize}")
    public ResponseBean queryMachineRoom(@PathVariable("page") @NotNull(message = "URL参数：当前页的页码不能为空") Integer page,
                                         @PathVariable("pageSize") @NotNull(message = "URL参数：当前页的页数不能为空") Integer pageSize,
                                         @RequestBody MachineRoom machineRoom) {
        return machineRoomService.queryMachineRoom(page - 1, pageSize, machineRoom.getName(), machineRoom.getProjectId());
    }

    @ApiOperation("视频大屏首页列表接口(项目下包括机房)")
    @PostMapping("/videoScreenList")
    public ResponseBean<VideoScreenListVo> videoScreenList() {
        return machineRoomService.videoScreenList();
    }

    @ApiOperation("获取萤石token")
    @GetMapping("/getYsToken")
    public ResponseBean getYsToken() {

        String ysAccessToken = null;
        try {
            ysAccessToken = cameraDeviceService.getYsAccessToken();
        } catch (Exception e) {
            log.error("获取萤石token失败:", e);
            return ResponseBean.fail("获取萤石token失败");
        }
        return ResponseBean.success(ysAccessToken);

    }

}
