package com.ebei.screen.service;

import com.ebei.screen.common.response.ResponseBean;
import com.ebei.screen.modules.req.QueryPageReq;
import com.ebei.screen.modules.req.RealTimeReq;

import java.util.Map;

/**
 * EBA系统大屏
 *
 * @author Levi.u
 * @date 2021/1/13 14:26
 */
public interface EbaSystemService {

    /**
     * 初始化数据
     */
    void initData();

    /**
     * 初始化当前用户名token到mongo
     */
    void initToken();

    /**
     * 初始化当前用户名下项目列表到mongo
     */
    void initProjectList();

    /**
     * 初始化设备信息(deviceSys)到mongo
     */
    void initDeviceSystemList();

    /**
     * 初始化设备类型信息(deviceType)到mongo
     */
    void initDeviceTypeList();

    /**
     * 初始化设备参数信息(deviceParam)到mongo
     */
    void initDeviceParamList();

    /**
     * 初始化设备告警类型信息(alarmType)到mongo
     */
    void initDeviceAlarmTypeList();

    /**
     * 初始化消防设备类型信息(fireDeviceType)到mongo
     */
    void initFireDeviceTypeList();

    /**
     * 初始化所有项目实时device信息到mongo
     */
    String initRealDataDeviceList(String key);

    /**
     * 初始化所有项目device信息到mongo
     */
    String initMobjectList(String key);

    /**
     * 初始化所有项目iot信息到mongo
     */
    String initIotList(String key);

    /**
     * 初始化楼上三个哥们
     */
    void initThreeBrothers();

    /**
     * 实时数据回调接口对查
     *
     * @param params
     * @return
     */
    Integer realTimeCallback(Map<String, String> params);

    /**
     * 获取项目列表
     *
     * @return
     */
    ResponseBean getProjectList();

    /**
     * 获取实时设备列表
     *
     * @param projectId 项目id
     * @return
     */
    ResponseBean getDeviceList(String projectId);

    /**
     * 获取实时火警和预警
     *
     * @param projectId 项目id
     * @return
     */
    ResponseBean getRealTimeList(String projectId);

    /**
     * 保存项目关联关系
     *
     * @param params
     * @return
     */
    ResponseBean relevanceProject(Map<String, String> params);

    /**
     * 查询已配置的crm项目列表
     *
     * @return
     */
    ResponseBean getShowProjectList();

    /**
     * 获取crm项目列表
     *
     * @return
     */
    ResponseBean getCrmProjectList(String key);

    /**
     * 分页查询所有已配置关联的项目列表
     *
     * @param pageBean
     * @return
     */
    ResponseBean getProjectListByPage(QueryPageReq pageBean);

    /**
     * 删除已有的关联项目
     *
     * @param ebeiProjectId
     * @return
     */
    ResponseBean deleteRelevance(String ebeiProjectId);

    /**
     * 分页查询火警和故障信息
     *
     * @param params
     * @return
     */
    ResponseBean getRealTimeListByPage(RealTimeReq params);

    /**
     * 根据文档名称修复火警故障数字
     *
     * @param name
     * @return
     */
    ResponseBean fixDate(String name);

    /**
     * 根据文档名称修复丢失天数
     *
     * @param name
     * @return
     */
    ResponseBean fix(String name);

    /**
     * 清除某个项目所有实时火警历史数据
     *
     * @param name
     * @return
     */
    ResponseBean delRealTimeByName(String name);

    /**
     * 恢复默认排序
     *
     * @param type
     * @return
     */
    ResponseBean resetSort(String type);
}
