package com.ebei.screen.service;

import com.ebei.screen.common.response.ResponseBean;
import com.ebei.screen.common.util.Levi;
import com.ebei.screen.modules.req.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Map;

/**
 * 车场系统大屏
 *
 * @author Levi.u
 * @date 2021/1/13 14:26
 */
public interface ParkSystemService {

    /**
     * 统一存放到mongo中后续处理
     *
     * @param id     业务id
     * @param params 参数
     * @return
     */
    Levi exec(String id, Map params, boolean flag);

    /**
     * 处理任务
     *
     * @param size 处理数量
     * @return
     */
    boolean taskExec(int size) throws InterruptedException;

    /**
     * 车场信息数据推送
     *
     * @param params
     * @return
     */
    void parkInfo(Map params);

    /**
     * 车位信息数据推送
     *
     * @param params
     * @return
     */
    void parkSpace(Map params);

    /**
     * 入场信息数据推送
     *
     * @param params
     * @return
     */
    void parkIn(Map params);

    /**
     * 出场信息数据推送
     *
     * @param params
     * @return
     */
    void parkOut(Map params);

    /**
     * 订单信息数据推送
     *
     * @param params
     * @return
     */
    void parkOrder(Map params);

    /**
     * 收费记录数据推送
     *
     * @param params
     * @return
     */
    void parkCharge(Map params);

    /**
     * 服务信息数据推送
     *
     * @param params
     */
    void parkServiceInfo(Map params);

    /**
     * 今日车场流量分析查询
     *
     * @param parkCode
     * @return
     */
    ResponseBean getTodayFlowAnalysis(String parkCode);

    /**
     * 今日车场流量分析分页查询详情
     *
     * @param params
     * @return
     */
    ResponseBean getTodayFlowAnalysisByPage(FlowAnalysisReq params);

    /**
     * 入场车辆通车时间段统计
     *
     * @param params
     * @return
     */
    ResponseBean inCarAnalysis(InCarAnalysisReq params);

    /**
     * 入场车辆通车时间段统计分页查询详情
     *
     * @param params
     * @return
     */
    ResponseBean inCarAnalysisByPage(InCarAnalysisPageReq params);

    /**
     * 今日停车时长统计
     *
     * @param parkCode
     * @return
     */
    ResponseBean parkingHoursToday(String parkCode);

    /**
     * 今日停车时长统计分页查询详情
     *
     * @param params
     * @return
     */
    ResponseBean parkingHoursTodayByPage(ParkHoursPageReq params);

    /**
     * 关键指标查询
     *
     * @param parkCode
     * @return
     */
    ResponseBean keyIndex(String parkCode);

    /**
     * 不同收费方式停车统计
     *
     * @param parkCode
     * @return
     */
    ResponseBean differentPay(String parkCode);

    /**
     * 不同收费方式停车统计分页查询详情
     *
     * @param params
     * @return
     */
    ResponseBean differentPayByPage(DifferentPayPageReq params);

    /**
     * 查询已配置的crm项目列表
     *
     * @return
     */
    ResponseBean getShowProjectList();

    /**
     * 删除已有的关联项目
     *
     * @param ebeiProjectId
     * @return
     */
    ResponseBean deleteRelevance(String ebeiProjectId);


    /**
     * 获取项目列表
     *
     * @return
     */
    ResponseBean getProjectList();

    /**
     * 今日车场流量分析分页查询折线
     *
     * @param params
     * @return
     */
    ResponseBean getTodayFlowAnalysisByLine(FlowAnalysisReq params);

    /**
     * 关键指标分页查询详情
     *
     * @param params
     * @return
     */
    ResponseBean keyIndexByPage(KeyIndexPageReq params);

    /**
     * 异常趋势统计查询
     *
     * @param parkCode
     * @return
     */
    ResponseBean exceptionAnalysis(String parkCode);

    /**
     * 异常趋势统计分页查询详情
     *
     * @param params
     * @return
     */
    ResponseBean exceptionAnalysisByPage(ExceptionTrendReq params);

    /**
     * 根据集合名称修复所有指定日期后的数字
     *
     * @param collection
     * @param date
     * @return
     */
    ResponseBean fixDate(String collection, String date);

    /**
     * 异常信息导出
     *
     * @param params
     * @param response
     */
    void exportException(ExceptionTrendReq params, HttpServletResponse response);
}
