package com.ebei.screen.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.ebei.screen.common.config.auth.LoginUtils;
import com.ebei.screen.common.constants.CommonConstants;
import com.ebei.screen.common.constants.EbaMessageCode;
import com.ebei.screen.common.constants.EbaSystemConstants;
import com.ebei.screen.common.response.PageBean;
import com.ebei.screen.common.response.ResponseBean;
import com.ebei.screen.common.response.ResponseBuilder;
import com.ebei.screen.common.util.*;
import com.ebei.screen.modules.req.QueryPageReq;
import com.ebei.screen.modules.req.RealTimeReq;
import com.ebei.screen.service.EbaSystemService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Supplier;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * EBA系统大屏
 *
 * @author Levi.u
 * @date 2021/1/13 14:30
 */
@Slf4j
@Service("ebaSystemService")
@Transactional(rollbackFor = {Exception.class})
public class EbaSystemServiceImpl implements EbaSystemService {

    @Autowired
    private MongoTemplate mongoTemplate;

    @Value("${comm.getProjectList}")
    private String getProjectList;

    private final long SECOND_TIME = 3L;


    /**
     * 初始化数据
     */
    @Override
    public void initData() {
        initToken();
        initProjectList();
//        initDeviceSystemList();
//        initDeviceTypeList();
//        initDeviceParamList();
//        initDeviceAlarmTypeList();
//        initFireDeviceTypeList();
//        initThreeBrothers();
        log.info("EBA数据全部初始化完毕");
    }

    /**
     * 初始化当前用户名token到mongo
     */
    @Override
    public void initToken() {
        String str = JSON.toJSONString(Levi.by("clientId", EbaSystemConstants.cliendId).set("clientSecret", EbaSystemConstants.clientSecret));
        String result = HttpRequest.post(EbaSystemConstants.getTokenInfo)
                .header("content-type", "application/json")
                .body(str)
                .execute().body();
        String token = LeviUtils.getJsonFieldOne(result, "data.token", String.class);
        Query query = new Query();
        query.addCriteria(Criteria.where("_id").is("token"));
        Update update = new Update();
        update.set("data", token);
        update.set("expire", LocalDateTime.now());
        mongoTemplate.upsert(query, update, "ebaAuth");
        EbaUtils.token = token;
    }

    /**
     * 初始化当前用户名下项目列表到mongo
     */
    @Override
    public void initProjectList() {
        String str = HttpRequest.post(EbaSystemConstants.getProjectList)
                .header("token", EbaUtils.token)
                .execute().body();
        if (EbaUtils.isSuccess(str)) {
            Query query = new Query();
            query.addCriteria(Criteria.where("_id").is("projectList"));
            Update update = new Update();
            List<Map> map = LeviUtils.getJsonFieldMany(str, "data.projectList", Map.class);
            JSONObject data = new JSONObject();
            map.forEach(x -> data.put(x.get("projectId").toString(), x));
            update.set("data", data);
            mongoTemplate.upsert(query, update, "ebaAuth");
            EbaUtils.project = data;
        }
    }

    /**
     * 初始化设备信息(deviceSys)到mongo
     */
    @Override
    public void initDeviceSystemList() {
        String str = HttpRequest.post(EbaSystemConstants.getDeviceSystemList)
                .header("token", EbaUtils.token)
                .execute().body();
        if (EbaUtils.isSuccess(str)) {
            Query query = new Query();
            query.addCriteria(Criteria.where("_id").is("deviceSystemList"));
            Update update = new Update();
            List<Map> map = LeviUtils.getJsonFieldMany(str, "data.datas", Map.class);
            JSONObject data = new JSONObject();
            map.forEach(x -> data.put(x.get("deviceSysId").toString(), x));
            update.set("data", data);
            mongoTemplate.upsert(query, update, "ebaAuth");
            EbaUtils.deviceSystem = data;
        }
    }

    /**
     * 初始化设备类型信息(deviceType)到mongo
     */
    @Override
    public void initDeviceTypeList() {
        String str = HttpRequest.post(EbaSystemConstants.getDeviceTypeList)
                .header("token", EbaUtils.token)
                .execute().body();
        if (EbaUtils.isSuccess(str)) {
            Query query = new Query();
            query.addCriteria(Criteria.where("_id").is("deviceTypeList"));
            Update update = new Update();
            List<Map> map = LeviUtils.getJsonFieldMany(str, "data.datas", Map.class);
            JSONObject data = new JSONObject();
            map.forEach(x -> data.put(x.get("deviceTypeId").toString(), x));
            update.set("data", data);
            mongoTemplate.upsert(query, update, "ebaAuth");
            EbaUtils.deviceType = data;
        }
    }

    /**
     * 初始化设备参数信息(deviceParam)到mongo
     */
    @Override
    public void initDeviceParamList() {
        String str = HttpRequest.post(EbaSystemConstants.getDeviceParamList)
                .header("token", EbaUtils.token)
                .execute().body();
        if (EbaUtils.isSuccess(str)) {
            Query query = new Query();
            query.addCriteria(Criteria.where("_id").is("deviceParamList"));
            Update update = new Update();
            List<Map> map = LeviUtils.getJsonFieldMany(str, "data.datas", Map.class);
            JSONObject data = new JSONObject();
            map.forEach(x -> data.put(x.get("deviceParamId").toString(), x));
            update.set("data", data);
            mongoTemplate.upsert(query, update, "ebaAuth");
            EbaUtils.deviceParam = data;
        }
    }

    /**
     * 初始化设备告警类型信息(alarmType)到mongo
     */
    @Override
    public void initDeviceAlarmTypeList() {
        String str = HttpRequest.post(EbaSystemConstants.getDeviceAlarmTypeList)
                .header("token", EbaUtils.token)
                .execute().body();
        if (EbaUtils.isSuccess(str)) {
            Query query = new Query();
            query.addCriteria(Criteria.where("_id").is("deviceAlarmTypeList"));
            Update update = new Update();
            List<Map> map = LeviUtils.getJsonFieldMany(str, "data.datas", Map.class);
            JSONObject data = new JSONObject();
            map.forEach(x -> data.put(x.get("alarmTypeId").toString(), x));
            update.set("data", data);
            mongoTemplate.upsert(query, update, "ebaAuth");
            EbaUtils.deviceAlarmType = data;
        }
    }

    /**
     * 初始化消防设备类型信息(fireDeviceType)到mongo
     */
    @Override
    public void initFireDeviceTypeList() {
        String str = HttpRequest.post(EbaSystemConstants.getFireDeviceTypeList)
                .header("token", EbaUtils.token)
                .execute().body();
        if (EbaUtils.isSuccess(str)) {
            Query query = new Query();
            query.addCriteria(Criteria.where("_id").is("fireDeviceTypeList"));
            Update update = new Update();
            List<Map> map = LeviUtils.getJsonFieldMany(str, "data.datas", Map.class);
            JSONObject data = new JSONObject();
            map.forEach(x -> data.put(x.get("fireDeviceTypeId").toString(), x));
            update.set("data", data);
            mongoTemplate.upsert(query, update, "ebaAuth");
            EbaUtils.fireDeviceType = data;
        }
    }

    /**
     * 初始化所有项目实时device信息到mongo
     */
    @Override
    public String initRealDataDeviceList(String key) {
        JSONObject data2 = new JSONObject();
        String param = JSON.toJSONString(Levi.by("projectId", Integer.parseInt(key)));
        String str = HttpRequest.post(EbaSystemConstants.getMobjectRealDataByProjectIdList)
                .header("token", EbaUtils.token)
                .body(param)
                .execute().body();
        if (EbaUtils.isSuccess(str)) {
            // 存入原始数据用于统计
            Query query1 = new Query();
            query1.addCriteria(Criteria.where("_id").is(key + "_" + LocalDateTime.now().getYear() + "_" + LocalDateTime.now().getMonthValue() + "_" + LocalDateTime.now().getDayOfMonth()));
            Update update1 = new Update();
            List<Map> map = LeviUtils.getJsonFieldMany(str, "data.realtimeDataList", Map.class);
            map.removeAll(Collections.singleton(null));
            update1.set("data", EbaUtils.setDataField(map, key, "realDataDeviceList", false, key + "_" + LocalDateTime.now().getYear() + "_" + LocalDateTime.now().getMonthValue() + "_" + LocalDateTime.now().getDayOfMonth()));
            mongoTemplate.upsert(query1, update1, "realDataDeviceList");
            // 存入当前最新数据方便查询
            JSONObject data1 = new JSONObject();
            map.forEach(x -> data1.put(x.get("deviceId").toString() + "_" + x.get("paramId").toString(), x.get("paramValue").toString()));
            data2.put(key, data1);
        }
        Query query2 = new Query();
        query2.addCriteria(Criteria.where("_id").is("realDataDeviceList"));
        Update update2 = new Update();
        update2.set("data", data2);
        mongoTemplate.upsert(query2, update2, "ebaAuth");
        EbaUtils.realDataDevice = data2;
        return str;
    }

    /**
     * 初始化所有项目device信息到mongo
     */
    @Override
    public String initMobjectList(String key) {
        String param = JSON.toJSONString(Levi.by("projectId", Integer.parseInt(key)));
        String str = HttpRequest.post(EbaSystemConstants.getMobjectList)
                .header("token", EbaUtils.token)
                .body(param)
                .execute().body();
        if (EbaUtils.isSuccess(str)) {
            Query query = new Query();
            query.addCriteria(Criteria.where("_id").is("mobjectList"));
            Update update = new Update();
            List<Map> map = LeviUtils.getJsonFieldMany(str, "data.mObjectList", Map.class);
            map.removeAll(Collections.singleton(null));
            JSONObject obj = mongoTemplate.findById("mobjectList", JSONObject.class, "ebaAuth");
            JSONObject data = obj != null ? obj.getJSONObject("data") : new JSONObject();
            map.forEach(x -> {
                List<Map> list = (List<Map>) x.get("params");
                Integer deviceId = (Integer) x.get("deviceId");
                if (CollUtil.isNotEmpty(list)) {
                    list.forEach(y -> y.put("paramValue", EbaUtils.realDataDevice.getJSONObject(key).getString(deviceId.toString() + "_" + y.get("paramId").toString())));
                }
            });
            data.put(key, map);
            update.set("data", data);
            mongoTemplate.upsert(query, update, "ebaAuth");
        }
        return str;
    }

    /**
     * 初始化所有项目iot信息到mongo
     */
    @Override
    public String initIotList(String key) {
        String param = JSON.toJSONString(Levi.by("projectId", Integer.parseInt(key)));
        String str = HttpRequest.post(EbaSystemConstants.getIotList)
                .header("token", EbaUtils.token)
                .body(param)
                .execute().body();
        if (EbaUtils.isSuccess(str)) {
            Query query = new Query();
            query.addCriteria(Criteria.where("_id").is("iotList"));
            JSONObject obj = mongoTemplate.findById("iotList", JSONObject.class, "ebaAuth");
            JSONObject data = obj != null ? obj.getJSONObject("data") : new JSONObject();
            Update update = new Update();
            List<Map> map = LeviUtils.getJsonFieldMany(str, "data.datas", Map.class);
            map.removeAll(Collections.singleton(null));
            data.put(key, map);
            update.set("data", data);
            mongoTemplate.upsert(query, update, "ebaAuth");
        }
        return str;
    }

    /**
     * 初始化楼上三个哥们
     */
    @Override
    public void initThreeBrothers() {
        Set<String> keys = EbaUtils.getProjectIds();
        keys.forEach(key -> ScheduledUtils.start(() -> {
            log.info("当前执行的项目id:{}", key);
            String s1 = initRealDataDeviceList(key);
            String s2 = initMobjectList(key);
            String s3 = initIotList(key);
            return Arrays.asList(s1, s2, s3);
        }, SECOND_TIME));
    }

    /**
     * 实时数据接收
     *
     * @param params
     * @return
     */
    @Override
    public Integer realTimeCallback(Map params) {
        // 判断类型
        String messageCode = params.get("messageCode").toString();
        String content = params.get("content").toString();
        List<Map> contentList = LeviUtils.getJsonFieldMany(content, null, Map.class);
        if (CollUtil.isEmpty(contentList)) {
            return 200;
        }
        List<Map> huojing = contentList.stream().filter(x -> ((x.get("AlarmType") == null ? -1 : (Integer) x.get("AlarmType")) == 1)).collect(Collectors.toList());
        List<Map> guzhang = contentList.stream().filter(x -> ((x.get("AlarmType") == null ? -1 : (Integer) x.get("AlarmType")) == 2)).collect(Collectors.toList());
        Query query = new Query();
        query.addCriteria(Criteria.where("_id").is(messageCode + "_" + LocalDateTime.now().getYear() + "_" + LocalDateTime.now().getMonthValue()));
        Update update = new Update();
        Map hjMap = EbaUtils.setDataFieldCount("hjCount", huojing, messageCode, "realTimeList", false);
        update.set("data", hjMap.get("data"));
        update.set("hjCount", hjMap.get("hjCount"));
        mongoTemplate.upsert(query, update, "realTimeList");
        Map gzMap = EbaUtils.setDataFieldCount("gzCount", guzhang, messageCode, "realTimeList", false);
        update.set("data", gzMap.get("data"));
        update.set("gzCount", gzMap.get("gzCount"));
        mongoTemplate.upsert(query, update, "realTimeList");
        return 200;
    }

    /**
     * 获取项目列表
     *
     * @return
     */
    @Override
    public ResponseBean getProjectList() {
        List<Map> results = new ArrayList<>();
        JSONObject map = mongoTemplate.findById("projectList", JSONObject.class, "ebaAuth");
        if (MapUtil.isNotEmpty(map)) {
            Map data = (Map) map.get("data");
            data.keySet().forEach(x -> results.add((Map) data.get(x)));
        }
        return ResponseBuilder.ok(results);
    }

    /**
     * 获取实时设备列表
     *
     * @param projectId 项目id
     * @return
     */
    @Override
    public ResponseBean getDeviceList(String projectId) {
        JSONObject obj = mongoTemplate.findById("mobjectList", JSONObject.class, "ebaAuth");
        JSONArray array = new JSONArray();
        if (StringUtils.isNotBlank(projectId)) {
            projectId = EbaUtils.getProjectId(projectId);
            array = obj.getJSONObject("data").getJSONArray(projectId);
        } else {
            JSONObject obj2 = mongoTemplate.findById("relevanceProject", JSONObject.class, "relevanceProject");
            JSONObject data = obj2 != null ? obj2.getJSONObject("data") : new JSONObject();
            List<String> collect = data.values().stream().map(x -> (String) x).distinct().collect(Collectors.toList());
            if (data != null) {
                for (String p : collect) {
                    String projectName = EbaUtils.project.getJSONObject(p) == null ? "" : EbaUtils.project.getJSONObject(p).getString("projectName");
                    if (obj.getJSONObject("data") != null && obj.getJSONObject("data").getJSONArray(p) != null) {
                        JSONArray xxx = obj.getJSONObject("data").getJSONArray(p);
                        xxx.stream().forEach(xx -> {
                            Map xObj = (Map) xx;
                            xObj.put("ProjectName", projectName);
                        });
                        array.addAll(obj.getJSONObject("data").getJSONArray(p));
                    }
                }
            }
        }
        if (array == null || array.size() <= 0) {
            return ResponseBuilder.ok(Levi.by("electricity", new JSONArray()).set("water", new JSONArray()));
        }
        List<Object> electricity = array.stream().filter(x -> {
            Map xx = (Map) x;
            return (Integer) xx.get("deviceSysId") == 1;
        }).collect(Collectors.toList());
        List<Object> water = array.stream().filter(x -> {
            Map xx = (Map) x;
            return (Integer) xx.get("deviceSysId") == 2;
        }).collect(Collectors.toList());
        return ResponseBuilder.ok(Levi.by("electricity", electricity).set("water", water));
    }

    /**
     * 获取实时火警和预警
     *
     * @param projectId 项目id
     * @return
     */
    @Override
    public ResponseBean getRealTimeList(String projectId) {
        boolean flag = StringUtils.isNotBlank(projectId);
        LocalDateTime now = LocalDateTime.now();
        int year = now.getYear();
        int month = now.getMonthValue();
        int day = now.getDayOfMonth();
        Map monthMap = mongoTemplate.findById(EbaMessageCode.D.getCode() + "_" + year + "_" + month, Map.class, "realTimeList");
        if (MapUtil.isEmpty(monthMap)) {
            return ResponseBuilder.ok("暂无实时火警和预警信息");
        }
        List<Map> dayList = (List<Map>) monthMap.get("data");
        Map dayMap = dayList.get(day - 1);
        List<Map> hourList = (List<Map>) dayMap.get("data");
        List<Map> filterList = hourList.stream().filter(x -> {
            List<Map> dataList = (List<Map>) x.get("data");
            return dataList.size() > 0;
        }).collect(Collectors.toList());
        // 抽取AlarmType:1火警、2故障
        List<Map> huojing = new ArrayList<>();
        List<Map> guzhang = new ArrayList<>();
        filterList.forEach(x -> {
            List<Map> dataList = (List<Map>) x.get("data");
            dataList.forEach(y -> {
                Integer alarmType = (Integer) y.get("AlarmType");
                Integer pid = (Integer) y.get("ProjectId");
                y.put("AlarmTimeStr", LeviUtils.convertTime(y.get("AlarmTime")));
                if (flag) {
                    if ((pid.toString()).equals(projectId)) {
                        if (alarmType == 1) {
                            huojing.add(y);
                        } else if (alarmType == 2) {
                            guzhang.add(y);
                        }
                    }
                } else {
                    if (alarmType == 1) {
                        huojing.add(y);
                    } else if (alarmType == 2) {
                        guzhang.add(y);
                    }
                }
            });
        });
        Integer hjDayCount = flag ? (Integer) dayMap.get("hjCount_" + projectId) : (Integer) dayMap.get("hjCount");
        Integer hjMonthCount = flag ? (Integer) monthMap.get("hjCount_" + projectId) : (Integer) monthMap.get("hjCount");
        Integer gzDayCount = flag ? (Integer) dayMap.get("gzCount_" + projectId) : (Integer) dayMap.get("gzCount");
        Integer gzMonthCount = flag ? (Integer) monthMap.get("gzCount_" + projectId) : (Integer) monthMap.get("gzCount");
        // 获取去年同时间数据
        LocalDateTime previousYear = now.minus(1, ChronoUnit.YEARS);
        Map oldMonthMap = mongoTemplate.findById(EbaMessageCode.D.getCode() + "_" + previousYear.getYear() + "_" + previousYear.getMonthValue(), Map.class, "realTimeList");
        if (MapUtil.isEmpty(oldMonthMap)) {
            return ResponseBuilder.ok(Levi.by("huojing", huojing).set("guzhang", guzhang).set("hjDayCount", hjDayCount).set("hjMonthCount", hjMonthCount).set("gzDayCount", gzDayCount).set("gzMonthCount", gzMonthCount).set("hjDayTb", "100%").set("hjMonthTb", "100%").set("gzDayTb", "100%").set("gzMonthTb", "100%"));
        }
        List<Map> oldDayList = (List<Map>) oldMonthMap.get("data");
        Map oldDayMap = oldDayList.get(day - 1);
        Integer hjDayCountOld = flag ? (Integer) oldDayMap.get("hjCount_" + projectId) : (Integer) oldDayMap.get("hjCount");
        Integer hjMonthCountOld = flag ? (Integer) oldMonthMap.get("hjCount_" + projectId) : (Integer) oldMonthMap.get("hjCount");
        Integer gzDayCountOld = flag ? (Integer) oldDayMap.get("gzCount_" + projectId) : (Integer) oldDayMap.get("gzCount");
        Integer gzMonthCountOld = flag ? (Integer) oldMonthMap.get("gzCount_" + projectId) : (Integer) oldMonthMap.get("gzCount");
        return ResponseBuilder.ok(Levi.by("huojing", huojing).set("guzhang", guzhang).set("hjDayCount", hjDayCount).set("hjMonthCount", hjMonthCount).set("gzDayCount", gzDayCount).set("gzMonthCount", gzMonthCount).set("hjDayTb", EbaUtils.getTongBi(hjDayCountOld, hjDayCount)).set("hjMonthTb", EbaUtils.getTongBi(hjMonthCountOld, hjMonthCount)).set("gzDayTb", EbaUtils.getTongBi(gzDayCountOld, gzDayCount)).set("gzMonthTb", EbaUtils.getTongBi(gzMonthCountOld, gzMonthCount)));
    }

    /**
     * 保存项目关联关系
     *
     * @param params
     * @return
     */
    @Override
    public ResponseBean relevanceProject(Map<String, String> params) {
        String type = params.get("type");
        // eba项目
        if (CommonConstants.EBA_TYPE.equals(type)) {
            JSONObject obj = mongoTemplate.findById("relevanceProject", JSONObject.class, "relevanceProject");
            JSONObject data = obj != null ? obj.getJSONObject("data") : new JSONObject();
            JSONObject obj2 = mongoTemplate.findById(CommonConstants.EBA_ORDER, JSONObject.class, CommonConstants.EBA_ORDER);
            List<String> crmIds = obj2 != null ? obj2.getJSONArray("data").toJavaList(String.class) : new JSONArray().toJavaList(String.class);
            List<String> collect = data.values().stream().map(x -> (String) x).distinct().collect(Collectors.toList());
            if (collect.contains(params.get("ebaId")) && !data.getString(params.get("crmId")).equals(params.get("ebaId"))) {
                return ResponseBuilder.fail("已关联 请重选");
            }
            if (StringUtils.isBlank(params.get("no")) || Integer.parseInt(params.get("no")) > crmIds.size() || Integer.parseInt(params.get("no")) <= 0) {
                return ResponseBuilder.fail("序号范围[1~" + crmIds.size() + "] 请重输");
            }
            if (StringUtils.isBlank(params.get("ebaId"))) {
                data.remove(params.get("crmId"));
            } else {
                data.put(params.get("crmId"), params.get("ebaId"));
            }
            Query query = new Query();
            query.addCriteria(Criteria.where("_id").is("relevanceProject"));
            Update update = new Update();
            update.set("data", data);
            mongoTemplate.upsert(query, update, "relevanceProject");
            // 旧索引
            int oldNo = crmIds.indexOf(params.get("crmId"));
            // 新索引
            int nowNo = Integer.parseInt(params.get("no")) - 1;
            if (oldNo != nowNo) {
                // 更新crm序号
                Collections.swap(crmIds, oldNo, nowNo);
                Query query2 = new Query();
                query2.addCriteria(Criteria.where("_id").is(CommonConstants.EBA_ORDER));
                Update update2 = new Update();
                update2.set("data", crmIds);
                mongoTemplate.upsert(query2, update2, CommonConstants.EBA_ORDER);
            }
            return ResponseBuilder.ok("eba项目关联成功");
        } else if (CommonConstants.ZHIHUI_TYPE.equals(type) || CommonConstants.WUYE_TYPE.equals(type)) {
            // 视频项目
            String key;
            if (CommonConstants.ZHIHUI_TYPE.equals(type)) {
                key = CommonConstants.ZHIHUI_ORDER;
            } else {
                key = CommonConstants.WUYE_ORDER;
            }
            JSONObject obj2 = mongoTemplate.findById(key, JSONObject.class, key);
            List<String> crmIds = obj2 != null ? obj2.getJSONArray("data").toJavaList(String.class) : new JSONArray().toJavaList(String.class);
            if (StringUtils.isBlank(params.get("no")) || Integer.parseInt(params.get("no")) > crmIds.size() || Integer.parseInt(params.get("no")) <= 0) {
                return ResponseBuilder.fail("序号范围[1~" + crmIds.size() + "] 请重输");
            }
            // 旧索引
            int oldNo = crmIds.indexOf(params.get("crmId"));
            // 新索引
            int nowNo = Integer.parseInt(params.get("no")) - 1;
            if (oldNo != nowNo) {
                // 更新crm序号
                Collections.swap(crmIds, oldNo, nowNo);
                Query query2 = new Query();
                query2.addCriteria(Criteria.where("_id").is(key));
                Update update2 = new Update();
                update2.set("data", crmIds);
                mongoTemplate.upsert(query2, update2, key);
            }
            return ResponseBuilder.ok("视频项目顺序调整成功");
        } else if (CommonConstants.PARK_TYPE.equals(type)) {
            // 车场项目
            JSONObject obj = mongoTemplate.findById("relevanceParkProject", JSONObject.class, "relevanceParkProject");
            JSONObject data = obj != null ? obj.getJSONObject("data") : new JSONObject();
            JSONObject obj2 = mongoTemplate.findById(CommonConstants.PARK_ORDER, JSONObject.class, CommonConstants.PARK_ORDER);
            List<String> crmIds = obj2 != null ? obj2.getJSONArray("data").toJavaList(String.class) : new JSONArray().toJavaList(String.class);
            List<String> collect = data.values().stream().map(x -> (String) x).distinct().collect(Collectors.toList());
            if (collect.contains(params.get("parkId")) && !data.getString(params.get("crmId")).equals(params.get("parkId"))) {
                return ResponseBuilder.fail("已关联 请重选");
            }
            if (StringUtils.isBlank(params.get("no")) || Integer.parseInt(params.get("no")) > crmIds.size() || Integer.parseInt(params.get("no")) <= 0) {
                return ResponseBuilder.fail("序号范围[1~" + crmIds.size() + "] 请重输");
            }
            if (StringUtils.isBlank(params.get("parkId"))) {
                data.remove(params.get("crmId"));
            } else {
                data.put(params.get("crmId"), params.get("parkId"));
            }
            Query query = new Query();
            query.addCriteria(Criteria.where("_id").is("relevanceParkProject"));
            Update update = new Update();
            update.set("data", data);
            mongoTemplate.upsert(query, update, "relevanceParkProject");
            // 旧索引
            int oldNo = crmIds.indexOf(params.get("crmId"));
            // 新索引
            int nowNo = Integer.parseInt(params.get("no")) - 1;
            if (oldNo != nowNo) {
                // 更新crm序号
                Collections.swap(crmIds, oldNo, nowNo);
                Query query2 = new Query();
                query2.addCriteria(Criteria.where("_id").is(CommonConstants.PARK_ORDER));
                Update update2 = new Update();
                update2.set("data", crmIds);
                mongoTemplate.upsert(query2, update2, CommonConstants.PARK_ORDER);
            }
            // 更新是否拆分 是拆分 否不拆分 默认否
            if (StringUtils.isNotBlank(params.get("split"))) {
                JSONObject obj3 = mongoTemplate.findById(CommonConstants.PARK_SPLIT, JSONObject.class, CommonConstants.PARK_SPLIT);
                List<String> parkCodeIds = obj3 != null ? obj3.getJSONArray("data").toJavaList(String.class) : new JSONArray().toJavaList(String.class);
                // 无脑删除 如果拆分则新增
                parkCodeIds.remove(params.get("parkId"));
                // 拆分则新增
                if ("是".equals(params.get("split"))) {
                    parkCodeIds.add(params.get("parkId"));
                }
                // 更新需要拆分的集合
                Query query3 = new Query();
                query3.addCriteria(Criteria.where("_id").is(CommonConstants.PARK_SPLIT));
                Update update3 = new Update();
                update3.set("data", parkCodeIds);
                mongoTemplate.upsert(query3, update3, CommonConstants.PARK_SPLIT);
            }
            return ResponseBuilder.ok("车场项目调整成功");
        } else {
            return ResponseBuilder.fail("未找到相关项目");
        }
    }

    /**
     * 查询已配置的crm项目列表
     *
     * @return
     */
    @Override
    public ResponseBean getShowProjectList() {
        JSONObject obj = mongoTemplate.findById("relevanceProject", JSONObject.class, "relevanceProject");
        JSONObject data = obj != null ? obj.getJSONObject("data") : new JSONObject();
        JSONObject crmProjectInfo = EbaUtils.crmProjectInfo;
        List<Map> list = new ArrayList<>();
        data.keySet().forEach(x -> list.add(Levi.by("projectId", x).set("projectName", crmProjectInfo.get(x)).set("ebaId", data.getString(x))));
        return ResponseBuilder.ok(EbaUtils.sortList(list));
    }

    /**
     * 获取crm项目列表
     *
     * @return
     */
    @Override
    public ResponseBean getCrmProjectList(String key) {
        if (!LoginUtils.isLogin()) {
            return ResponseBuilder.ok(new ArrayList<>());
        }
        String str = HttpRequest.get(getProjectList + LoginUtils.getUserId())
                .execute().body();
        List<Map> projectLists = LeviUtils.getJsonFieldMany(str, "items", Map.class);
        JSONObject data = new JSONObject();
        projectLists.forEach(x -> data.put(x.get("projectId").toString(), x.get("projectName").toString()));
        EbaUtils.crmProjectInfo = data;
        List<String> nowCrmProjectSort = new ArrayList<>();
        if (CommonConstants.EBA_ORDER.equals(key)) {
            nowCrmProjectSort = EbaUtils.getNowCrmProjectSort(projectLists, CommonConstants.EBA_ORDER);
        } else if (CommonConstants.ZHIHUI_ORDER.equals(key)) {
            nowCrmProjectSort = EbaUtils.getNowCrmProjectSort(projectLists, CommonConstants.ZHIHUI_ORDER);
        } else if (CommonConstants.WUYE_ORDER.equals(key)) {
            nowCrmProjectSort = EbaUtils.getNowCrmProjectSort(projectLists, CommonConstants.WUYE_ORDER);
        } else if (CommonConstants.PARK_ORDER.equals(key)) {
            nowCrmProjectSort = EbaUtils.getNowCrmProjectSort(projectLists, CommonConstants.PARK_ORDER);
        }
        List<String> finalNowCrmProjectSort = nowCrmProjectSort;
        Collections.sort(projectLists, Comparator.comparingInt(o -> CollUtil.isNotEmpty(finalNowCrmProjectSort) ? finalNowCrmProjectSort.indexOf(o.get("projectId").toString()) : 0));
        return ResponseBuilder.ok(projectLists);
    }

    /**
     * 分页查询所有已配置关联的项目列表
     *
     * @param pageBean
     * @return
     */
    @Override
    public ResponseBean getProjectListByPage(QueryPageReq pageBean) {
        String type = pageBean.getType();
        Levi<String, String> parkInfo = ParkUtils.initParkCodeList();
        JSONObject crmProjectInfo = EbaUtils.crmProjectInfo;
        if (CommonConstants.EBA_TYPE.equals(type)) {
            JSONObject obj = mongoTemplate.findById("relevanceProject", JSONObject.class, "relevanceProject");
            JSONObject data = obj != null ? obj.getJSONObject("data") : new JSONObject();
            List<Map> list = new ArrayList<>();
            JSONObject obj2 = mongoTemplate.findById(CommonConstants.EBA_ORDER, JSONObject.class, CommonConstants.EBA_ORDER);
            List<String> ebeiProjects = obj2 != null ? obj2.getJSONArray("data").toJavaList(String.class) : new JSONArray().toJavaList(String.class);
            AtomicInteger i = new AtomicInteger(1);
            ebeiProjects.forEach(x -> list.add(Levi.by("ebaProjectId", data.get(x) == null ? "" : data.get(x)).set("ebaProjectName", data.get(x) == null ? "" : EbaUtils.project.getJSONObject(data.get(x).toString()) == null ? "" : EbaUtils.project.getJSONObject(data.get(x).toString()).getString("projectName")).set("ebeiProjectId", x).set("ebeiProjectName", crmProjectInfo.getString(x)).set("no", i.getAndIncrement())));
            Supplier<Stream<Map>> resStream = () -> list.stream();
            List<Map> results = resStream.get().skip((pageBean.getPageNum() - 1) * pageBean.getPageSize()).limit(pageBean.getPageSize()).collect(Collectors.toList());
            return ResponseBuilder.ok(PageBean.<Map>builder().total(resStream.get().count()).row(results).build());
        } else if (CommonConstants.ZHIHUI_TYPE.equals(type) || CommonConstants.WUYE_TYPE.equals(type)) {
            String key;
            if (CommonConstants.ZHIHUI_TYPE.equals(type)) {
                key = CommonConstants.ZHIHUI_ORDER;
            } else {
                key = CommonConstants.WUYE_ORDER;
            }
            List<Map> list = new ArrayList<>();
            JSONObject obj2 = mongoTemplate.findById(key, JSONObject.class, key);
            List<String> ebeiProjects = obj2 != null ? obj2.getJSONArray("data").toJavaList(String.class) : new JSONArray().toJavaList(String.class);
            AtomicInteger i = new AtomicInteger(1);
            ebeiProjects.forEach(x -> list.add(Levi.by("ebeiProjectId", x).set("ebeiProjectName", crmProjectInfo.getString(x)).set("no", i.getAndIncrement())));
            Supplier<Stream<Map>> resStream = () -> list.stream();
            List<Map> results = resStream.get().skip((pageBean.getPageNum() - 1) * pageBean.getPageSize()).limit(pageBean.getPageSize()).collect(Collectors.toList());
            return ResponseBuilder.ok(PageBean.<Map>builder().total(resStream.get().count()).row(results).build());
        } else if (CommonConstants.PARK_TYPE.equals(type)) {
            JSONObject obj = mongoTemplate.findById("relevanceParkProject", JSONObject.class, "relevanceParkProject");
            JSONObject data = obj != null ? obj.getJSONObject("data") : new JSONObject();
            List<Map> list = new ArrayList<>();
            JSONObject obj2 = mongoTemplate.findById(CommonConstants.PARK_ORDER, JSONObject.class, CommonConstants.PARK_ORDER);
            List<String> ebeiProjects = obj2 != null ? obj2.getJSONArray("data").toJavaList(String.class) : new JSONArray().toJavaList(String.class);
            JSONObject obj3 = mongoTemplate.findById(CommonConstants.PARK_SPLIT, JSONObject.class, CommonConstants.PARK_SPLIT);
            List<String> splitProjects = obj3 != null ? obj3.getJSONArray("data").toJavaList(String.class) : new JSONArray().toJavaList(String.class);
            AtomicInteger i = new AtomicInteger(1);
            ebeiProjects.forEach(x -> list.add(Levi.by("parkProjectId", data.get(x) == null ? "" : data.get(x)).set("parkProjectName", data.get(x) == null ? "" : parkInfo.get(data.get(x).toString())).set("ebeiProjectId", x).set("ebeiProjectName", crmProjectInfo.getString(x)).set("no", i.getAndIncrement()).set("split", data.get(x) == null ? "否" : (splitProjects.contains(data.get(x)) ? "是" : "否"))));
            Supplier<Stream<Map>> resStream = () -> list.stream();
            List<Map> results = resStream.get().skip((pageBean.getPageNum() - 1) * pageBean.getPageSize()).limit(pageBean.getPageSize()).collect(Collectors.toList());
            return ResponseBuilder.ok(PageBean.<Map>builder().total(resStream.get().count()).row(results).build());
        } else {
            return ResponseBuilder.fail("未找到相关项目");
        }
    }

    /**
     * 删除已有的关联项目
     *
     * @param ebeiProjectId
     * @return
     */
    @Override
    public ResponseBean deleteRelevance(String ebeiProjectId) {
        JSONObject obj = mongoTemplate.findById("relevanceProject", JSONObject.class, "relevanceProject");
        JSONObject data = obj != null ? obj.getJSONObject("data") : new JSONObject();
        data.remove(ebeiProjectId);
        Query query = new Query();
        query.addCriteria(Criteria.where("_id").is("relevanceProject"));
        Update update = new Update();
        update.set("data", data);
        mongoTemplate.upsert(query, update, "relevanceProject");
        return ResponseBuilder.ok("删除成功");
    }

    /**
     * 分页查询火警和故障信息
     *
     * @param params
     * @return
     */
    @Override
    public ResponseBean getRealTimeListByPage(RealTimeReq params) {
        String projectId = params.getProjectId();
        boolean flag = StringUtils.isNotBlank(projectId);
        Integer pageNum = params.getPageNum();
        Integer pageSize = params.getPageSize();
        Integer dateType = params.getDateType();
        Integer type = params.getType();
        LocalDateTime now = LocalDateTime.now();
        int year = now.getYear();
        int month = now.getMonthValue();
        int day = now.getDayOfMonth();
        Map monthMap = mongoTemplate.findById(EbaMessageCode.D.getCode() + "_" + year + "_" + month, Map.class, "realTimeList");
        if (MapUtil.isEmpty(monthMap)) {
            return ResponseBuilder.ok("暂无实时火警和预警信息");
        }
        List<Map> dayList = (List<Map>) monthMap.get("data");
        // 抽取AlarmType:1火警、2故障
        List<Map> huojing = new ArrayList<>();
        List<Map> guzhang = new ArrayList<>();
        // 查询一天 则替换集合元素为当日
        if (dateType == 0) {
            dayList = Arrays.asList(dayList.get(day - 1));
        }
        dayList.forEach(dayMap -> {
            List<Map> hourList = (List<Map>) dayMap.get("data");
            List<Map> filterList = hourList.stream().filter(x -> {
                List<Map> dataList = (List<Map>) x.get("data");
                return dataList.size() > 0;
            }).collect(Collectors.toList());
            filterList.forEach(x -> {
                List<Map> dataList = (List<Map>) x.get("data");
                dataList.forEach(y -> {
                    Integer alarmType = (Integer) y.get("AlarmType");
                    Integer pid = (Integer) y.get("ProjectId");
                    y.put("AlarmTimeStr", LeviUtils.convertTime(y.get("AlarmTime")));
                    if (flag) {
                        if ((pid.toString()).equals(projectId)) {
                            if (alarmType == 1) {
                                huojing.add(y);
                            } else if (alarmType == 2) {
                                guzhang.add(y);
                            }
                        }
                    } else {
                        if (alarmType == 1) {
                            huojing.add(y);
                        } else if (alarmType == 2) {
                            guzhang.add(y);
                        }
                    }
                });
            });
        });
        // 过滤多余字段 返回结果集
        if (type == 0) {
            List<Levi> hjList = huojing.stream().map(x -> Levi.by("ProjectName", x.get("ProjectName").toString()).set("ProjectId", x.get("ProjectId").toString()).set("AlarmContent", x.get("AlarmContent").toString()).set("AlarmTimeStr", x.get("AlarmTimeStr").toString())).collect(Collectors.toList());
            Supplier<Stream<Levi>> hjStream = () -> hjList.stream();
            List<Map> hjResult = hjStream.get().skip((pageNum - 1) * pageSize).limit(pageSize).collect(Collectors.toList());
            return ResponseBuilder.ok(PageBean.<Map>builder().total(hjStream.get().count()).row(hjResult).build());
        } else {
            List<Levi> gzList = guzhang.stream().map(x -> Levi.by("ProjectName", x.get("ProjectName").toString()).set("ProjectId", x.get("ProjectId").toString()).set("AlarmContent", x.get("AlarmContent").toString()).set("AlarmTimeStr", x.get("AlarmTimeStr").toString())).collect(Collectors.toList());
            Supplier<Stream<Levi>> gzStream = () -> gzList.stream();
            List<Map> gzResult = gzStream.get().skip((pageNum - 1) * pageSize).limit(pageSize).collect(Collectors.toList());
            return ResponseBuilder.ok(PageBean.<Map>builder().total(gzStream.get().count()).row(gzResult).build());
        }
    }

    /**
     * 根据文档名称修复火警故障数字
     *
     * @param name
     * @return
     */
    @Override
    public ResponseBean fixDate(String name) {
        Map monthMap = mongoTemplate.findById(name, Map.class, "realTimeList");
        Query query = new Query();
        query.addCriteria(Criteria.where("_id").is(name));
        Update update = new Update();
        List<Map> dayList = (List<Map>) monthMap.get("data");
        Map<String, Integer> zzz = new HashMap<>(16);
        dayList.forEach(dayMap -> {
            List<Map> hourList = (List<Map>) dayMap.get("data");
            Map<String, Integer> yyy = new HashMap<>(16);
            hourList.forEach(hourMap -> {
                List<Map> dataList = (List<Map>) hourMap.get("data");
                Map<String, Integer> xxx = new HashMap<>(16);
                for (Map x : dataList) {
                    String projectId = x.get("ProjectId").toString();
                    Integer alarmType = (Integer) x.get("AlarmType");
                    if (alarmType == 1 && xxx.get("hjCount_" + projectId) == null) {
                        xxx.put("hjCount_" + projectId, 1);
                        xxx.put("hjCount", xxx.get("hjCount") == null ? 1 : xxx.get("hjCount") + 1);
                    } else if (alarmType == 1 && xxx.get("hjCount_" + projectId) != null) {
                        xxx.put("hjCount_" + projectId, xxx.get("hjCount_" + projectId) + 1);
                        xxx.put("hjCount", xxx.get("hjCount") == null ? 1 : xxx.get("hjCount") + 1);
                    } else if (alarmType == 2 && xxx.get("gzCount_" + projectId) == null) {
                        xxx.put("gzCount_" + projectId, 1);
                        xxx.put("gzCount", xxx.get("gzCount") == null ? 1 : xxx.get("gzCount") + 1);
                    } else if (alarmType == 2 && xxx.get("gzCount_" + projectId) != null) {
                        xxx.put("gzCount_" + projectId, xxx.get("gzCount_" + projectId) + 1);
                        xxx.put("gzCount", xxx.get("gzCount") == null ? 1 : xxx.get("gzCount") + 1);
                    }
                }
                for (String key : xxx.keySet()) {
                    hourMap.put(key, xxx.get(key));
                    yyy.put(key, yyy.get(key) == null ? xxx.get(key) : yyy.get(key) + xxx.get(key));
                }
            });
            for (String key : yyy.keySet()) {
                dayMap.put(key, yyy.get(key));
                zzz.put(key, zzz.get(key) == null ? yyy.get(key) : zzz.get(key) + yyy.get(key));
            }
        });
        for (String key : zzz.keySet()) {
            update.set(key, zzz.get(key));
        }
        update.set("data", monthMap.get("data"));
        mongoTemplate.upsert(query, update, "realTimeList");
        Map xxx = mongoTemplate.findById(name, Map.class, "realTimeList");
        return ResponseBuilder.ok("修复完成", xxx);
    }

    /**
     * 根据文档名称修复丢失天数
     *
     * @param name
     * @return
     */
    @Override
    public ResponseBean fix(String name) {
        Map monthMap = mongoTemplate.findById(name, Map.class, "realTimeList");
        Query query = new Query();
        query.addCriteria(Criteria.where("_id").is(name));
        Update update = new Update();
        List<Map> monthList = (List<Map>) monthMap.get("data");
        int monthDays = LeviUtils.getMonthDays(LocalDateTime.now().getMonthValue());
        int count = 0;
        for (int i = monthList.size(); i < monthDays; i++) {
            List<Map> dayList = new ArrayList<>();
            for (int c = 0; c <= 23; c++) {
                dayList.add(Levi.by("hour", c).set("data", new ArrayList<>()));
            }
            monthList.add(Levi.by("day", i + 1).set("data", dayList));
            count++;
        }
        update.set("data", monthMap.get("data"));
        mongoTemplate.upsert(query, update, "realTimeList");
        return ResponseBuilder.ok("累计修复" + count + "天成功");
    }

    /**
     * 清除某个项目所有实时火警历史数据
     *
     * @param name
     * @return
     */
    @Override
    public ResponseBean delRealTimeByName(String name) {
        List<Map> realTimeList = mongoTemplate.findAll(Map.class, "realTimeList");
        List<Map> ids = realTimeList.stream().filter(x -> x.get("_id").toString().contains("1003_")).collect(Collectors.toList());
        ids.forEach(monthMap -> {
            Query query = new Query();
            query.addCriteria(Criteria.where("_id").is(monthMap.get("_id").toString()));
            Update update = new Update();
            List<Map> dayList = (List<Map>) monthMap.get("data");
            dayList.forEach(dayMap -> {
                List<Map> hourList = (List<Map>) dayMap.get("data");
                hourList.forEach(hourMap -> {
                    List<Map> dataList = (List<Map>) hourMap.get("data");
                    Iterator<Map> it = dataList.iterator();
                    while (it.hasNext()) {
                        Map x = it.next();
                        String projectId = x.get("ProjectId").toString();
                        String projectName = x.get("ProjectName").toString();
                        if (projectId.equals(name) || projectName.equals(name)) {
                            it.remove();
                        }
                    }
                });
            });
            update.set("data", monthMap.get("data"));
            mongoTemplate.upsert(query, update, "realTimeList");
        });
        Map xxx = mongoTemplate.findById(name, Map.class, "realTimeList");
        return ResponseBuilder.ok("删除完成", xxx);
    }

    /**
     * 恢复默认排序
     *
     * @param type
     * @return
     */
    @Override
    public ResponseBean resetSort(String type) {
        if (!LoginUtils.isLogin()) {
            return ResponseBuilder.ok(new ArrayList<>());
        }
        String str = HttpRequest.get(getProjectList + LoginUtils.getUserId())
                .execute().body();
        List<Map> projectLists = LeviUtils.getJsonFieldMany(str, "items", Map.class);
        if (CommonConstants.EBA_TYPE.equals(type)) {
            EbaUtils.resetSortList(CommonConstants.EBA);
            EbaUtils.getNowCrmProjectSort(projectLists, CommonConstants.EBA_ORDER);
        } else if (CommonConstants.ZHIHUI_TYPE.equals(type) || CommonConstants.WUYE_TYPE.equals(type)) {
            String key;
            String keyOrder;
            if (CommonConstants.ZHIHUI_TYPE.equals(type)) {
                key = CommonConstants.ZHIHUI;
                keyOrder = CommonConstants.ZHIHUI_ORDER;
            } else {
                key = CommonConstants.WUYE;
                keyOrder = CommonConstants.WUYE_ORDER;
            }
            EbaUtils.resetSortList(key);
            EbaUtils.getNowCrmProjectSort(projectLists, keyOrder);
        } else if (CommonConstants.PARK_TYPE.equals(type)) {
            EbaUtils.resetSortList(CommonConstants.PARK);
            EbaUtils.getNowCrmProjectSort(projectLists, CommonConstants.PARK_ORDER);
        }
        return ResponseBuilder.ok("重置成功");
    }
}
