package com.ebei.screen.service.impl;

import com.ebei.screen.common.constants.CommonConstants;
import com.ebei.screen.common.response.ResponseBean;
import com.ebei.screen.common.util.CommonUtil;
import com.ebei.screen.common.util.EbaUtils;
import com.ebei.screen.modules.po.CameraDevice;
import com.ebei.screen.modules.po.MachineRoom;
import com.ebei.screen.modules.po.Project;
import com.ebei.screen.modules.po.VideoScreenListVo;
import com.ebei.screen.service.EbaSystemService;
import com.ebei.screen.service.MachineRoomService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.*;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.*;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @version v1.0
 * @ProjectName: screen
 * @ClassName: MachineRoomServiceImpl
 * @Description:
 * @Author: ch
 * @Date: 2021-01-20 14:07
 */
@Slf4j
@Service
@AllArgsConstructor
public class MachineRoomServiceImpl implements MachineRoomService {

    @Autowired
    private MongoTemplate mongoTemplate;
    @Autowired
    private EbaSystemService ebaSystemService;

    @Override
    public void saveMachineRoom(MachineRoom machineRoom) {
        LocalDateTime now = LocalDateTime.now();
        machineRoom.setId(CommonUtil.getSnowId().toString());
        machineRoom.setCreateTime(now);
        machineRoom.setType(machineRoom.getType());
        machineRoom.setIsDelete(0);
        if (machineRoom.getType().equals(1) && CommonUtil.isNotEmpty(machineRoom.getCameraDeviceList()) && CommonUtil.isNotEmpty(machineRoom.getVerCode())) {
            //监控设备拼接上验证码
            machineRoom.getCameraDeviceList().forEach(i -> {
                if (i.getUrl().contains("888888")) {
                    String url = i.getUrl().replace("888888", machineRoom.getVerCode());
                    i.setUrl(url);
                }
            });
        }
        mongoTemplate.insert(machineRoom);
    }

    @Override
    public void updateMachineRoom(MachineRoom machineRoom) {
        LocalDateTime now = LocalDateTime.now();
        Query query = new Query();
        query.addCriteria(Criteria.where("_id").is(machineRoom.getId()));
        Update update = new Update();
        update.set("name", machineRoom.getName());
        update.set("projectId", machineRoom.getProjectId());
        update.set("projectName", machineRoom.getProjectName());
        update.set("type", machineRoom.getType());
        if (machineRoom.getType().equals(1) && CommonUtil.isNotEmpty(machineRoom.getCameraDeviceList()) && CommonUtil.isNotEmpty(machineRoom.getVerCode())) {
            //监控设备拼接上验证码
            machineRoom.getCameraDeviceList().forEach(i -> {
                if (i.getUrl().contains("888888")) {
                    String url = i.getUrl().replace("888888", machineRoom.getVerCode());
                    i.setUrl(url);
                }
            });
        }
        update.set("cameraDeviceList", machineRoom.getCameraDeviceList());
        update.set("updateTime", now);
        update.set("verCode", machineRoom.getVerCode());
        update.set("isDelete", machineRoom.getIsDelete());
        mongoTemplate.updateFirst(query, update, MachineRoom.class);
    }

    @Override
    public ResponseBean queryMachineRoom(int page, Integer pageSize, String name, String projectId) {
        Query query = new Query();
        if (CommonUtil.isNotEmpty(name)) {
            Criteria criteria = new Criteria();
            Pattern pattern = Pattern.compile("^.*" + name + ".*$", Pattern.CASE_INSENSITIVE);
            criteria.where("name").regex(pattern);
            query.addCriteria(criteria);
        }
        if (CommonUtil.isNotEmpty(projectId)) {
            query.addCriteria(Criteria.where("projectId").is(projectId));
        }
        query.addCriteria(Criteria.where("isDelete").is(0));
        //计算总数
        long total = mongoTemplate.count(query, MachineRoom.class, "machineRoom");
        //分页查询
        Pageable pageable = PageRequest.of(page, pageSize);
        query.with(pageable);
        query.with(Sort.by(Sort.Direction.DESC, "createTime"));
        List<MachineRoom> result = mongoTemplate.find(query, MachineRoom.class);
        Page<MachineRoom> pageResult = new PageImpl<>(result, pageable, total);
        return ResponseBean.success(pageResult);
    }

    @Override
    public ResponseBean videoScreenList() {
        //查询所有项目
        ResponseBean crmProjectList = ebaSystemService.getCrmProjectList(null);
        List<Map> projects = (List<Map>) crmProjectList.getData();
        if (CommonUtil.isEmpty(projects)) {
            return ResponseBean.success();
        }
        List<Project> ysProject = new ArrayList<>();
        List<Project> gldProject = new ArrayList<>();
        //查询项目下关联的机房、摄像头
        projects.forEach(i -> {
            //查询项目下是否关联机房、机房下是否关联摄像头
            Query query = new Query();
            String projectId = String.valueOf(i.get("projectId"));
            String projectName = String.valueOf(i.get("projectName"));
            query.addCriteria(Criteria.where("projectId").is(projectId));
            query.addCriteria(Criteria.where("isDelete").is(0));
            List<MachineRoom> machineRooms = mongoTemplate.find(query, MachineRoom.class);
            if (CommonUtil.isNotEmpty(machineRooms)) {
                machineRooms.forEach(machineRoom -> {
                    List<CameraDevice> cameraList = machineRoom.getCameraDeviceList();
                    CameraDevice cameraDevice = cameraList.get(0);
                    cameraDevice.setMachineRoomName(machineRoom.getName());
                    if (machineRoom.getType().equals(1)) {
                        //萤石设备
                        List<Project> exist = ysProject.stream().filter(x -> x.getId().equals(projectId)).collect(Collectors.toList());
                        if (CommonUtil.isEmpty(exist)) {
                            //不存在项目
                            Project project = new Project();
                            project.setId(projectId);
                            project.setName(projectName);
                            ArrayList<CameraDevice> ysCameraDeviceList = new ArrayList<>();
                            ysCameraDeviceList.add(cameraDevice);
                            project.setCameraList(ysCameraDeviceList);
                            ysProject.add(project);
                        } else {
                            //存在
                            Project project = exist.get(0);
                            project.getCameraList().add(cameraDevice);
                        }
                    } else {
                        //广联达设备
                        List<Project> exist = gldProject.stream().filter(x -> x.getId().equals(projectId)).collect(Collectors.toList());
                        if (CommonUtil.isEmpty(exist)) {
                            //不存在项目
                            Project project = new Project();
                            project.setId(projectId);
                            project.setName(projectName);
                            ArrayList<CameraDevice> ysCameraDeviceList = new ArrayList<>();
                            ysCameraDeviceList.add(cameraDevice);
                            project.setCameraList(ysCameraDeviceList);
                            gldProject.add(project);
                        } else {
                            //存在
                            Project project = exist.get(0);
                            project.getCameraList().add(cameraDevice);
                        }
                    }
                });
            }
        });
        VideoScreenListVo result = new VideoScreenListVo();
        // 对项目进行排序
        List<String> sortList1 = EbaUtils.getNowCrmProjectSort(projects, CommonConstants.ZHIHUI_ORDER);
        List<String> sortList2 = EbaUtils.getNowCrmProjectSort(projects, CommonConstants.WUYE_ORDER);
        Collections.sort(gldProject, Comparator.comparingInt(o -> sortList1.indexOf(o.getId())));
        Collections.sort(ysProject, Comparator.comparingInt(o -> sortList2.indexOf(o.getId())));
        result.setGldList(gldProject);
        result.setYsList(ysProject);
        return ResponseBean.success(result);
    }
}
